# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

from openstack import exceptions

from otcextensions.tests.functional import base


class TestVlb(base.BaseFunctionalTest):
    uuid_v4 = uuid.uuid4().hex[:8]
    network = None
    load_balancer = None
    listener = None
    additional_listener = None
    pool = None
    member = None
    health_monitor = None
    certificate = None
    l7policy = None
    l7rule = None
    server = None
    keypair = None
    ip_address_group = None
    security_policy = None

    _private_key = """-----BEGIN RSA PRIVATE KEY-----
MIICXQIBAAKBgQDFPN9ojPndxSC4E1pqWQVKGHCFlXAAGBOxbGfSzXqzsoyacotu
eqMqXQbxrPSQFATeVmhZPNVEMdvcAMjYsV/mymtAwVqVA6q/OFdX/b3UHO+b/VqL
o3J5SrM86Veqnjzwu4oCSabuEDiN+tga1syQmEG4OFM6NSmAYSxcZdE6LwIDAQAB
AoGBAJvLzJCyIsCJcKHWL6onbSUtDtyFwPViD1QrVAtQYabF14g8CGUZG/9fgheu
TXPtTDcvu7cZdUArvgYW3I9F9IBb2lmF3a44xfiAKdDhzr4DK/vQhvHPuuTeZA41
r2zp8Cu+Bp40pSxmoAOK3B0/peZAka01Ju7c7ZChDWrxleHZAkEA/6dcaWHotfGS
eW5YLbSms3f0m0GH38nRl7oxyCW6yMIDkFHURVMBKW1OhrcuGo8u0nTMi5IH9gRg
5bH8XcujlQJBAMWBQgzCHyoSeryD3TFieXIFzgDBw6Ve5hyMjUtjvgdVKoxRPvpO
kclc39QHP6Dm2wrXXHEej+9RILxBZCVQNbMCQQC42i+Ut0nHvPuXN/UkXzomDHde
h1ySsOAO4H+8Y6OSI87l3HUrByCQ7stX1z3L0HofjHqV9Koy9emGTFLZEzSdAkB7
Ei6cUKKmztkYe3rr+RcATEmwAw3tEJOHmrW5ErApVZKr2TzLMQZ7WZpIPzQRCYnY
2ZZLDuZWFFG3vW+wKKktAkAaQ5GNzbwkRLpXF1FZFuNF7erxypzstbUmU/31b7tS
i5LmxTGKL/xRYtZEHjya4Ikkkgt40q1MrUsgIYbFYMf2
-----END RSA PRIVATE KEY-----"""

    _certificate = """-----BEGIN CERTIFICATE-----
MIIDIjCCAougAwIBAgIJALV96mEtVF4EMA0GCSqGSIb3DQEBBQUAMGoxCzAJBgNV
BAYTAnh4MQswCQYDVQQIEwJ4eDELMAkGA1UEBxMCeHgxCzAJBgNVBAoTAnh4MQsw
CQYDVQQLEwJ4eDELMAkGA1UEAxMCeHgxGjAYBgkqhkiG9w0BCQEWC3h4eEAxNjMu
Y29tMB4XDTE3MTExMzAyMjYxM1oXDTIwMTExMjAyMjYxM1owajELMAkGA1UEBhMC
eHgxCzAJBgNVBAgTAnh4MQswCQYDVQQHEwJ4eDELMAkGA1UEChMCeHgxCzAJBgNV
BAsTAnh4MQswCQYDVQQDEwJ4eDEaMBgGCSqGSIb3DQEJARYLeHh4QDE2My5jb20w
gZ8wDQYJKoZIhvcNAQEBBQADgY0AMIGJAoGBAMU832iM+d3FILgTWmpZBUoYcIWV
cAAYE7FsZ9LNerOyjJpyi256oypdBvGs9JAUBN5WaFk81UQx29wAyNixX+bKa0DB
WpUDqr84V1f9vdQc75v9WoujcnlKszzpV6qePPC7igJJpu4QOI362BrWzJCYQbg4
Uzo1KYBhLFxl0TovAgMBAAGjgc8wgcwwHQYDVR0OBBYEFMbTvDyvE2KsRy9zPq/J
WOjovG+WMIGcBgNVHSMEgZQwgZGAFMbTvDyvE2KsRy9zPq/JWOjovG+WoW6kbDBq
MQswCQYDVQQGEwJ4eDELMAkGA1UECBMCeHgxCzAJBgNVBAcTAnh4MQswCQYDVQQK
EwJ4eDELMAkGA1UECxMCeHgxCzAJBgNVBAMTAnh4MRowGAYJKoZIhvcNAQkBFgt4
eHhAMTYzLmNvbYIJALV96mEtVF4EMAwGA1UdEwQFMAMBAf8wDQYJKoZIhvcNAQEF
BQADgYEAASkC/1iwiALa2RU3YCxqZFEEsZZvQxikrDkDbFeoa6Tk49Fnb1f7FCW6
PTtY3HPWl5ygsMsSy0Fi3xp3jmuIwzJhcQ3tcK5gC99HWp6Kw37RL8WoB8GWFU0Q
4tHLOjBIxkZROPRhH+zMIrqUexv6fsb3NWKhnlfh1Mj5wQE4Ldo=
-----END CERTIFICATE-----"""

    def setUp(self):
        super(TestVlb, self).setUp()
        self.client = self.conn.vlb
        self.net_client = self.conn.network
        self.ecs_client = self.conn.compute

    def create_load_balancer(
            self,
            az_list=None,
            publicip: dict = None,
            tags: list = None,
            name='sdk-vlb-test-lb-' + uuid_v4,
            description='test',
            guaranteed=True,
            provider='vlb',
            ip_target_enable=True,
            **kwargs
    ):
        attrs = {
            'name': name,
            'description': description,
            'vip_subnet_cidr_id': TestVlb.network['subnet_id'],
            'vpc_id': TestVlb.network['router_id'],
            'elb_virsubnet_ids': [TestVlb.network['subnet_id']],
            'guaranteed': guaranteed,
            'provider': provider,
            'availability_zone_list': az_list,
            'publicip': publicip,
            'ip_target_enable': ip_target_enable,
            'tags': tags,
            **kwargs
        }
        if not az_list:
            attrs['availability_zone_list'] = ['eu-de-01']
        if not publicip:
            attrs['publicip'] = {
                "network_type": "5_bgp",
                "billing_info": "",
                "bandwidth": {
                    "size": 2,
                    "share_type": "PER",
                    "charge_mode": "traffic",
                    "name": "elbv3_eip_traffic"
                }
            }
        if not tags:
            attrs['tags'] = [{
                "key": "test",
                "value": "api"
            }]
        if TestVlb.network and not TestVlb.load_balancer:
            TestVlb.load_balancer = self.client.create_load_balancer(**attrs)

    def create_listener(
            self,
            insert_headers: dict = None,
            name='sdk-vlb-test-lis-' + uuid_v4,
            protocol_port=80,
            protocol='TCP',
            tags: list = None,
            additional=False,
            **kwargs
    ):
        attrs = {
            'protocol_port': protocol_port,
            'protocol': protocol,
            'insert_headers': insert_headers,
            'name': name,
            'tags': tags,
            **kwargs
        }
        if not TestVlb.load_balancer:
            raise exceptions.SDKException
        attrs['loadbalancer_id'] = TestVlb.load_balancer.id
        if not insert_headers:
            attrs['insert_headers'] = {'X-Forwarded-ELB-IP': True}
        if not tags:
            attrs['tags'] = [{
                "key": "test",
                "value": "api"
            }]
        if TestVlb.network and TestVlb.load_balancer \
                and not TestVlb.listener:
            TestVlb.listener = self.client.create_listener(**attrs)
        if additional and not TestVlb.additional_listener:
            TestVlb.additional_listener = self.client.create_listener(**attrs)

    def create_pool(
            self,
            admin_state_up=True,
            description='Test',
            lb_algorithm='ROUND_ROBIN',
            name='sdk-vlb-test-pool-' + uuid_v4,
            protocol='TCP',
            **kwargs
    ):
        attrs = {
            'name': name,
            'description': description,
            'lb_algorithm': lb_algorithm,
            'protocol': protocol,
            'admin_state_up': admin_state_up,
            **kwargs
        }
        if not TestVlb.listener:
            raise exceptions.SDKException
        attrs['listener_id'] = TestVlb.listener.id
        if not TestVlb.load_balancer:
            raise exceptions.SDKException
        attrs['loadbalancer_id'] = TestVlb.load_balancer.id
        if TestVlb.network and TestVlb.listener and not TestVlb.pool:
            TestVlb.pool = self.client.create_pool(**attrs)

    def create_member(
            self,
            name='sdk-vlb-test-member-' + uuid_v4,
            protocol_port=8080,
            **kwargs
    ):
        attrs = {
            'protocol_port': protocol_port,
            'name': name,
            'address': TestVlb.server.addresses[self.network_name][0]['addr'],
            **kwargs
        }
        if TestVlb.pool and not TestVlb.member:
            TestVlb.member = self.client.create_member(
                TestVlb.pool, **attrs)

    def create_health_monitor(
            self,
            type='TCP',
            timeout=3,
            delay=5,
            max_retries=3,
            admin_state_up=True,
            monitor_port=8080,
            **kwargs
    ):
        attrs = {
            'type': type,
            'timeout': timeout,
            'delay': delay,
            'max_retries': max_retries,
            'admin_state_up': admin_state_up,
            'monitor_port': monitor_port,
            **kwargs
        }
        if not TestVlb.pool:
            raise exceptions.SDKException
        attrs['pool_id'] = TestVlb.pool.id
        if TestVlb.pool and not TestVlb.health_monitor:
            TestVlb.health_monitor = self.client.create_health_monitor(**attrs)

    def create_certificate(
            self,
            private_key=_private_key,
            certificate=_certificate,
            name='sdk-vlb-test-cert-' + uuid_v4,
            **kwargs
    ):
        if not TestVlb.certificate:
            TestVlb.certificate = self.client.create_certificate(
                private_key=private_key,
                certificate=certificate,
                name=name,
                **kwargs
            )

    def create_l7policy(
            self,
            redirect_listener_id,
            action='REDIRECT_TO_LISTENER',
            name='sdk-vlb-test-l7p-' + uuid_v4,
            **kwargs
    ):
        attrs = {
            'action': action,
            'name': name,
            'redirect_listener_id': redirect_listener_id,
            **kwargs
        }
        if not TestVlb.listener:
            raise exceptions.SDKException
        attrs['listener_id'] = TestVlb.listener.id
        if TestVlb.listener and not TestVlb.l7policy:
            TestVlb.l7policy = self.client.create_l7_policy(**attrs)

    def create_l7rule(
            self,
            compare_type='EQUAL_TO',
            admin_state_up=True,
            value='/test.com',
            type='PATH',
            **kwargs
    ):
        attrs = {
            'compare_type': compare_type,
            'admin_state_up': admin_state_up,
            'value': value,
            'type': type,
            **kwargs
        }
        if TestVlb.l7policy and not TestVlb.l7rule:
            TestVlb.l7rule = self.client.create_l7_rule(
                TestVlb.l7policy,
                **attrs
            )

    def create_ip_address_group(
            self,
            name='test_ip_group-' + uuid_v4,
            description='Test',
            **kwargs
    ):
        attrs = {
            'name': name,
            'description': description,
            'ip_list': [{"ip": "192.168.1.123", "description": ""}],
            **kwargs
        }
        if not TestVlb.ip_address_group:
            TestVlb.ip_address_group = self.client.create_ip_address_group(
                **attrs)

    def create_security_policy(
            self,
            name='test_security_policy-' + uuid_v4,
            description='Test',
            **kwargs
    ):
        attrs = {
            'name': name,
            'description': description,
            'protocols': ["TLSv1.2", "TLSv1", "TLSv1.3"],
            'ciphers': ["ECDHE-ECDSA-AES128-SHA", "TLS_AES_128_GCM_SHA256"],
            **kwargs
        }
        if not TestVlb.security_policy:
            TestVlb.security_policy = self.client.create_security_policy(
                **attrs)

    def create_server(
            self,
            name='sdk-vlb-test-ecs-' + uuid_v4,
            kp_name='sdk-vlb-test-kp-' + uuid_v4,
            image_name='Standard_Fedora_34_latest',
            flavor_name='s3.medium.1'
    ):

        image = self.ecs_client.find_image(image_name)
        flavor = self.ecs_client.find_flavor(flavor_name)
        self.network_name = self.net_client.find_network(
            TestVlb.network['network_id']
        ).name

        if not TestVlb.keypair:
            TestVlb.keypair = self.ecs_client.create_keypair(name=kp_name)

        if not TestVlb.server:
            server = self.ecs_client.create_server(
                name=name,
                image_id=image.id,
                flavor_id=flavor.id,
                networks=[{"uuid": TestVlb.network['network_id']}],
                key_name=TestVlb.keypair.name
            )

            TestVlb.server = self.ecs_client.wait_for_server(server)

    def create_network(
            self,
            cidr='192.168.0.0/16',
            ip_version=4,
            router_name='sdk-vlb-test-router-',
            net_name='sdk-vlb-test-net-',
            subnet_name='sdk-vlb-test-subnet-'
    ):
        uuid_v4 = uuid.uuid4().hex[:8]
        router_name += uuid_v4
        net_name += uuid_v4
        subnet_name += uuid_v4
        if not TestVlb.network:
            network = self.conn.network.create_network(name=net_name)
            self.assertEqual(net_name, network.name)
            net_id = network.id
            subnet = self.conn.network.create_subnet(
                name=subnet_name,
                ip_version=ip_version,
                network_id=net_id,
                cidr=cidr
            )
            self.assertEqual(subnet_name, subnet.name)
            subnet_id = subnet.id

            router = self.conn.network.create_router(name=router_name)
            self.assertEqual(router_name, router.name)
            router_id = router.id
            interface = router.add_interface(
                self.conn.network,
                subnet_id=subnet_id
            )
            self.assertEqual(interface['subnet_id'], subnet_id)
            self.assertIn('port_id', interface)
            TestVlb.network = {
                'router_id': router_id,
                'subnet_id': subnet_id,
                'network_id': net_id
            }
        return

    def destroy_network(self, params: dict):
        router_id = params.get('router_id')
        subnet_id = params.get('subnet_id')
        network_id = params.get('network_id')
        router = self.conn.network.get_router(router_id)

        interface = router.remove_interface(
            self.conn.network,
            subnet_id=subnet_id
        )
        self.assertEqual(interface['subnet_id'], subnet_id)
        self.assertIn('port_id', interface)
        sot = self.conn.network.delete_router(
            router_id,
            ignore_missing=False
        )
        self.assertIsNone(sot)
        sot = self.conn.network.delete_subnet(
            subnet_id,
            ignore_missing=False
        )
        self.assertIsNone(sot)
        sot = self.conn.network.delete_network(
            network_id,
            ignore_missing=False
        )
        self.assertIsNone(sot)

    def delete_server(self):
        sot = self.ecs_client.delete_server(TestVlb.server.id)
        self.assertIsNone(sot)
        self.ecs_client.wait_for_delete(TestVlb.server)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock

from otcextensions.osclient.vpcep.v1 import whitelist
from otcextensions.tests.unit.osclient.vpcep.v1 import fakes


class TestListWhitelist(fakes.TestVpcep):

    _service = fakes.FakeService.create_one()
    objects = fakes.FakeWhitelist.create_multiple(3)
    column_list_headers = ('Id', 'Permission', 'Created At')

    columns = (
        'id',
        'persmission',
        'created_at',
    )

    data = []

    for s in objects:
        data.append(
            (
                s.id,
                s.permission,
                s.created_at,
            )
        )

    def setUp(self):
        super(TestListWhitelist, self).setUp()

        self.cmd = whitelist.ListWhitelist(self.app, None)

        self.client.find_service = mock.Mock(return_value=self._service)
        self.client.service_whitelist = mock.Mock()
        self.client.api_mock = self.client.service_whitelist

    def test_list(self):
        arglist = [self._service.name]

        verifylist = [
            ('service', self._service.name),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(self._service)

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            self._service.name,
            '--sort-key',
            'created_at',
            '--sort-dir',
            'asc',
            '--limit',
            '2',
            '--offset',
            '3',
        ]

        verifylist = [
            ('service', self._service.name),
            ('sort_key', 'created_at'),
            ('sort_dir', 'asc'),
            ('limit', 2),
            ('offset', 3),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            self._service,
            sort_key='created_at',
            sort_dir='asc',
            limit=2,
            offset=3,
        )


class TestManageWhitelist(fakes.TestVpcep):

    _service = fakes.FakeService.create_one()
    objects = fakes.FakeWhitelist.create_multiple(3)
    column_list_headers = ('Permission',)

    columns = ('persmission',)

    data = []

    for s in objects:
        data.append((s.permission,))

    def setUp(self):
        super(TestManageWhitelist, self).setUp()

        self.cmd = whitelist.ManageWhitelist(self.app, None)

        self.client.find_service = mock.Mock(return_value=self._service)
        self.client.manage_service_whitelist = mock.Mock()
        self.client.api_mock = self.client.manage_service_whitelist

    def test_add_whitelist(self):
        arglist = [self._service.name, '123', 'xyz', '--add']

        verifylist = [
            ('service', self._service.name),
            ('domain', ['123', 'xyz']),
            ('add', True),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            self._service,
            domains=['123', 'xyz'],
            action='add',
        )

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_remove_whitelist(self):
        arglist = [self._service.name, '123', 'xyz', '--remove']

        verifylist = [
            ('service', self._service.name),
            ('domain', ['123', 'xyz']),
            ('remove', True),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            self._service,
            domains=['123', 'xyz'],
            action='remove',
        )

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import exceptions


class QuotaProxyMixin:

    @staticmethod
    def find_value_by_accessor(input_dict, accessor):
        """Gets value from a dictionary using a dotted accessor"""
        current_data = input_dict
        for chunk in accessor.split('.'):
            if isinstance(current_data, dict):
                current_data = current_data.get(chunk, {})
            else:
                return None
        return current_data

    @classmethod
    def list(cls, session, paginated=False,
             base_path=None, **params):
        if not cls.allow_list:
            raise exceptions.MethodNotSupported(cls, "list")

        session = cls._get_session(session)

        # pop scaling_group_id, as it should not be also present in the query
        scaling_group_id = params.pop('scaling_group_id', None)
        uri_params = {}

        if scaling_group_id:
            uri_params = {'scaling_group_id': scaling_group_id}

        cls._query_mapping._validate(params, base_path=cls.base_path)
        query_params = cls._query_mapping._transpose(params, cls)
        uri = cls.base_path % uri_params

        limit = query_params.get('limit')

        total_yielded = 0
        while uri:
            response = session.get(
                uri,
                params=query_params.copy()
            )
            exceptions.raise_from_response(response)
            data = response.json()

            # Discard any existing pagination keys
            query_params.pop('marker', None)
            query_params.pop('limit', None)

            if cls.resources_key:
                resources = cls.find_value_by_accessor(data, cls.resources_key)
            else:
                resources = data

            if not isinstance(resources, list):
                resources = [resources]

            marker = None
            for raw_resource in resources:
                # Do not allow keys called "self" through. Glance chose
                # to name a key "self", so we need to pop it out because
                # we can't send it through cls.existing and into the
                # Resource initializer. "self" is already the first
                # argument and is practically a reserved word.
                raw_resource.pop("self", None)

                if cls.resource_key and cls.resource_key in raw_resource:
                    raw_resource = raw_resource[cls.resource_key]

                value = cls.existing(**raw_resource)

                marker = value.id
                yield value
                total_yielded += 1

            if resources and paginated:
                uri, next_params = cls._get_next_link(
                    uri, response, data, marker, limit, total_yielded)
                query_params.update(next_params)
            else:
                return



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.function_graph.v2 import import_function
from otcextensions.sdk.function_graph.v2 import export_function

EXAMPLE_EXPORT = {
    'file_name': 'test',
    'code_url': 'url'
}
EXAMPLE_IMPORT = {
    "func_urn": "urn",
    "func_name": "test_v1_2",
    "domain_id": "14ee2e35****a7998b******aa24cabf",
    "project_name": "{region}",
    "package": "default",
    "runtime": "Node.js6.10",
    "timeout": 3,
    "handler": "index.handler",
    "memory_size": 128,
    "cpu": 300,
    "code_type": "zip",
    "code_filename": "index.zip",
    "code_size": 6709,
    "digest": "123",
    "version": "latest",
    "image_name": "latest-191025153727@zehht",
    "last_modified": "2019-10-25 15:37:27",
    "strategy_config": {
        "concurrency": -1
    },
    "enterprise_project_id": "123"
}


class TestFunctionExport(base.TestCase):

    def test_basic(self):
        sot = export_function.Export()
        path = '/fgs/functions/%(function_urn)s/export'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_fetch)

    def test_make_it(self):
        sot = export_function.Export(**EXAMPLE_EXPORT)
        self.assertEqual(EXAMPLE_EXPORT['file_name'], sot.file_name)
        self.assertEqual(EXAMPLE_EXPORT['code_url'], sot.code_url)


class TestFunctionImport(base.TestCase):

    def test_basic(self):
        sot = import_function.Import()
        path = '/fgs/functions/import'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_create)

    def test_make_it(self):
        sot = import_function.Import(**EXAMPLE_IMPORT)
        self.assertEqual(EXAMPLE_IMPORT['func_name'], sot.func_name)
        self.assertEqual(EXAMPLE_IMPORT['code_type'], sot.code_type)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''LoadBalancer Pool Member v1 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)

_formatters = {

}


def _get_columns(item):
    column_map = {
        'is_admin_state_up': 'admin_state_up',
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListPoolMember(command.Lister):
    _description = _('List members in a pool')
    columns = (
        'id', 'name', 'project_id', 'provisioning_status', 'address',
        'protocol_port', 'operating_status', 'weight')

    def get_parser(self, prog_name):
        parser = super(ListPoolMember, self).get_parser(prog_name)

        parser.add_argument(
            'pool',
            metavar='<pool>',
            help=_('Pool name or ID to list the members of.')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Load balancer pool member name to query')
        )
        parser.add_argument(
            '--address',
            metavar='<address>',
            help=_('Load balancer pool member address to query')
        )
        parser.add_argument(
            '--protocol_port',
            metavar='<protocol_port>',
            type=int,
            help=_('Load balancer pool member port number to query')
        )
        parser.add_argument(
            '--weight',
            metavar='<weight>',
            type=int,
            help=_('Load balancer pool member weight to query')
        )

        return parser

    def take_action(self, parsed_args):

        args = {}
        if parsed_args.address:
            args['address'] = parsed_args.address
        if parsed_args.protocol_port:
            args['protocol_port'] = parsed_args.protocol_port
        if parsed_args.name:
            args['name'] = parsed_args.name
        if parsed_args.weight:
            args['weight'] = parsed_args.weight

        client = self.app.client_manager.network

        data = client.pool_members(pool=parsed_args.pool, **args)

        return (
            self.columns,
            (utils.get_item_properties(
                s, self.columns
            ) for s in data))


class ShowPoolMember(command.ShowOne):
    _description = _('Shows details of a single Member')

    def get_parser(self, prog_name):
        parser = super(ShowPoolMember, self).get_parser(prog_name)

        parser.add_argument(
            'pool',
            metavar='<pool>',
            help=_('Pool name or ID to show the members of.')
        )
        parser.add_argument(
            'member',
            metavar='<member>',
            help=_('Name or ID of the member to show.')
        )

        return parser

    def take_action(self, parsed_args):

        args = {}

        client = self.app.client_manager.network

        obj = client.find_pool_member(
            name_or_id=parsed_args.member,
            pool=parsed_args.pool,
            ignore_missing=False,
            **args)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class CreatePoolMember(command.ShowOne):
    _description = _('Creating a member in a pool')

    def get_parser(self, prog_name):
        parser = super(CreatePoolMember, self).get_parser(prog_name)

        parser.add_argument(
            'pool',
            metavar='<pool>',
            help=_('ID or name of the pool to create the member for.')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Name of the member.')
        )
        parser.add_argument(
            '--weight',
            metavar='<weight>',
            type=int,
            choices=range(0, 256),
            help='The weight of a member determines the portion of requests '
                 'or connections it services compared to the other members of '
                 'the pool.'
        )
        parser.add_argument(
            '--address',
            metavar='<ip_address>',
            help=_('The IP address of the backend member to receive traffic '
                   'from the load balancer.')
        )
        parser.add_argument(
            '--protocol_port',
            metavar='<protocol_port>',
            type=int,
            choices=range(1, 65535),
            help=_('The port on which the backend member listens for traffic.')
        )
        parser.add_argument(
            '--subnet_id',
            metavar='<subnet_id>',
            help=_('The subnet ID the member service is accessible from.')
        )
        admin_group = parser.add_mutually_exclusive_group()
        admin_group.add_argument(
            '--enable',
            action='store_true',
            default=True,
            help=_('Enable member (default).')
        )
        admin_group.add_argument(
            '--disable',
            action='store_true',
            default=None,
            help=_('Disable member.')
        )
        return parser

    def take_action(self, parsed_args):

        args = {}
        args['address'] = parsed_args.address
        if parsed_args.name:
            args['name'] = parsed_args.name
        if parsed_args.protocol_port:
            args['protocol_port'] = parsed_args.protocol_port
        if parsed_args.subnet_id:
            args['subnet_id'] = parsed_args.subnet_id
        if isinstance(parsed_args.weight, int):
            args['weight'] = parsed_args.weight
        if parsed_args.disable:
            args['is_admin_state_up'] = False

        client = self.app.client_manager.network

        obj = client.create_pool_member(
            pool=parsed_args.pool, **args)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class SetPoolMember(command.ShowOne):
    _description = _('Update a member')
    columns = ('ID', 'Name', 'address', 'is_admin_state_up',
               'protocol_port', 'operating_status',
               'subnet_id', 'weight', 'pool_id')

    def get_parser(self, prog_name):
        parser = super(SetPoolMember, self).get_parser(prog_name)

        parser.add_argument(
            'pool',
            metavar='<pool>',
            help=_('Pool name or ID to show the members of.')
        )
        parser.add_argument(
            'member',
            metavar='<member>',
            help=_('Name or ID of the member to show.')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Name or ID of the member to update.')
        )
        parser.add_argument(
            '--weight',
            metavar='<weight>',
            type=int,
            choices=range(0, 256),
            help=_('The weight of a member determines the portion of requests '
                   'or connections it services compared to the other members '
                   'of the pool. For example, a member with a weight of 10 '
                   'receives five times as many requests as a member with a '
                   'weight of 2. A value of 0 means the member does not '
                   'receive new connections but continues to service existing '
                   'connections. A valid value is from 0 to 256. '
                   'Default is 1.')
        )
        admin_group = parser.add_mutually_exclusive_group()
        admin_group.add_argument(
            '--enable',
            action='store_true',
            default=True,
            help=_('Enable member (default).')
        )
        admin_group.add_argument(
            '--disable',
            action='store_true',
            default=None,
            help=_('Disable member.')
        )
        return parser

    def take_action(self, parsed_args):

        args = {}
        if parsed_args.disable:
            args['is_admin_state_up'] = False
        if parsed_args.name:
            args['name'] = parsed_args.name
        if isinstance(parsed_args.weight, int):
            args['weight'] = parsed_args.weight

        client = self.app.client_manager.network

        obj = client.update_pool_member(
            pool_member=parsed_args.member,
            pool=parsed_args.pool,
            **args)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class DeletePoolMember(command.Command):
    _description = _('Delete a member from a pool')

    def get_parser(self, prog_name):
        parser = super(DeletePoolMember, self).get_parser(prog_name)

        parser.add_argument(
            'pool',
            metavar='<pool>',
            help=_('Pool name or ID to delete the member from.')
        )
        parser.add_argument(
            'member',
            metavar='<member>',
            nargs='+',
            help=_('Name or ID of the member to be deleted.')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.network

        for member in parsed_args.member:
            obj = client.find_pool_member(name_or_id=member,
                                          pool=parsed_args.pool,
                                          ignore_missing=False)
            client.delete_pool_member(
                pool_member=obj.id,
                pool=parsed_args.pool,
                ignore_missing=False
            )

        return



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.load_balancer.v2 import _proxy

from otcextensions.common.utils import extract_url_parts
from otcextensions.sdk.elb.v2 import elb_certificate as _certificate
from otcextensions.sdk.elb.v2 import load_balancer as _load_balancer
from openstack.load_balancer.v2 import listener as _listener
from otcextensions.sdk.elb.v2 import load_balancer_tag as _lb_tag
from otcextensions.sdk.elb.v2 import listener_tag as _lstnr_tag


class Proxy(_proxy.Proxy):
    skip_discovery = True

    def _extract_name(self, url, service_type=None, project_id=None):
        return extract_url_parts(url, project_id)

    # ======== Certificate ========
    def create_certificate(self, **attrs):
        """Create a new certificate from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.elb.v2.elb_certificate.Certificate`,
            comprised of the properties on the Certificate class.

        :returns: The results of the Certificate Creation

        :rtype: :class:`~otcextensions.sdk.elb.v2.elb_certificate.Certificate`
        """
        return self._create(_certificate.Certificate, **attrs)

    def certificates(self, **query):
        """Return a generator of certificates

        :param dict query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of certificates objects.
        """
        return self._list(_certificate.Certificate, **query)

    def delete_certificate(self, certificate, ignore_missing=True):
        """Delete a certificate

        :param certificate: The value can be the ID of a ELB certificate or a
            :class:`~otcextensions.sdk.elb.v2.elb_certificate.Certificate`
             instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
             when the certificate does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent certificate.

        :returns: ``None``
        """
        return self._delete(_certificate.Certificate, certificate,
                            ignore_missing=ignore_missing)

    def get_certificate(self, certificate):
        """Get a single certificate

        :param certificate: The value can be the ID of a ELB certificate or a
            :class:`~otcextensions.sdk.elb.v2.elb_certificate.Certificate`
            instance.

        :returns: One :class:
        `~otcextensions.sdk.elb.v2.elb_certificate.Certificate`

        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_certificate.Certificate, certificate)

    def update_certificate(self, certificate, **attrs):
        """Update a certificate

        :param certificate: The value can be either the ID of a ELB certificate
         or a :class:`~otcextensions.sdk.elb.v2.elb_certificate.Certificate`
          instance.
        :param dict attrs: The attributes to update on the certificate
         represented by ``certificate``.

        :returns: The updated certificate.

        :rtype: :class:`~otcextensions.elb.v2.elb_certificate.Certificate`
        """
        return self._update(_certificate.Certificate, certificate, **attrs)

    def find_certificate(self, name_or_id, ignore_missing=False):
        """Find a single certificate

        :param name_or_id: The name or ID of a ELB certificate
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the certificate does not exist.
            When set to ``True``, no exception will be set when attempting
            to find a nonexistent certificate.

        :returns:
            One :class:`~otcextensions.sdk.elb.v2.elb_certificate.Certificate`
             or ``None``
        """
        return self._find(_certificate.Certificate, name_or_id,
                          ignore_missing=ignore_missing)

    # ======== Load Balancer ========
    def delete_loadbalancer(self, load_balancer, ignore_missing=True,
                            cascade=False):
        """Delete load balancer

        :param load_balancer: Either the ID of a load_balancer
         or a :class:`~otcextensions.sdk.elb.v2.load_balancer.LoadBalancer`
         instance.

        :returns: ``None``
        """
        loadbalancer = self.find_load_balancer(name_or_id=load_balancer)
        if cascade:
            resources = self.process_resources(loadbalancer)
            for healthmonitor in resources['healthmonitors']:
                self.delete_health_monitor(healthmonitor)
            for member in resources['members']:
                self.delete_member(
                    member=member['member_id'],
                    pool=member['pool_id']
                )
            for pool in resources['pools']:
                self.delete_pool(pool)
            for listener in resources['listeners']:
                self.delete_listener(listener)

        return self._delete(
            _load_balancer.LoadBalancer, load_balancer,
            ignore_missing=ignore_missing)

    def process_resources(self, loadbalancer):
        resources = {
            'listeners': [],
            'pools': [],
            'members': [],
            'healthmonitors': []
        }
        if loadbalancer.get('listeners'):
            for listener in loadbalancer.listeners:
                resources['listeners'].append(listener['id'])
        if loadbalancer.get('pools'):
            for pool in loadbalancer.pools:
                resources['pools'].append(pool['id'])
                find_pool = self.find_pool(name_or_id=pool['id'])
                if find_pool.get('health_monitor_id'):
                    resources['healthmonitors'].append(
                        find_pool['health_monitor_id'])
                if find_pool.get('members'):
                    for member in find_pool['members']:
                        resources['members'].append(
                            {
                                'member_id': member['id'],
                                'pool_id': find_pool['id']
                            }
                        )
        return resources

    # ======== Load Balancer Tag ========
    def load_balancer_tags(self, load_balancer, **query):
        """Return a generator of tags

        :param load_balancer: The load_balancer can be either the ID of a
            load balancer or
            :class:`~otcextensions.sdk.elb.v2.loadbalancer.Loadbalancer`
            instance that the load_balancer belongs to.
        :param dict query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of tags objects.
        """
        lb_obj = self._get_resource(
            _load_balancer.LoadBalancer,
            load_balancer
        )
        pr_id = self.session.get_project_id()
        base_path = pr_id + _lb_tag.Tag.base_path
        return self._list(
            _lb_tag.Tag,
            base_path=base_path,
            loadbalancer_id=lb_obj.id,
            paginated=False,
            **query)

    def create_load_balancer_tag(self, load_balancer, **attrs):
        """Create a new tag from attributes

        :param load_balancer: The load_balancer can be either the ID of a
            load balancer or
            :class:`~otcextensions.sdk.elb.v2.loadbalancer.Loadbalancer`
            instance that the load_balancer belongs to.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.elb.v2.load_balancer_tag.Tag`,
            comprised of the properties on the Tag class.

        :returns: The results of the Tag Creation

        :rtype: :class:`~otcextensions.sdk.elb.v2.load_balancer_tag.Tag`
        """
        lb_obj = self._get_resource(
            _load_balancer.LoadBalancer,
            load_balancer
        )
        return self._create(
            _lb_tag.Tag,
            loadbalancer_id=lb_obj.id,
            **attrs)

    def delete_load_balancer_tag(
            self, load_balancer, key, ignore_missing=True
    ):
        """Delete a tag

        :param key: tag key
        :param load_balancer: The load_balancer can be either the ID of a
            load balancer or
            :class:`~otcextensions.sdk.elb.v2.loadbalancer.Loadbalancer`
            instance that the load_balancer belongs to..
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the tag does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent tag.

        :returns: ``None``
        """
        lb_obj = self._get_resource(
            _load_balancer.LoadBalancer,
            load_balancer
        )
        return self._delete(
            _lb_tag.Tag,
            key,
            loadbalancer_id=lb_obj.id,
            ignore_missing=ignore_missing,
        )

    # ======== Listener Tag ========
    def listener_tags(self, listener, **query):
        """Return a generator of tags

        :param listener: The listener can be either the ID of a
            listener or
            :class:`~otcextensions.sdk.elb.v2.listener.Listener`
            instance that the listener belongs to.
        :param dict query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of tags objects.
        """
        listener_obj = self._get_resource(
            _listener.Listener,
            listener
        )
        pr_id = self.session.get_project_id()
        base_path = pr_id + _lstnr_tag.Tag.base_path
        return self._list(
            _lstnr_tag.Tag,
            listener_id=listener_obj.id,
            base_path=base_path,
            paginated=False,
            **query)

    def create_listener_tag(self, listener, **attrs):
        """Create a new tag from attributes

        :param listener: The listener can be either the ID of a
            listener or
            :class:`~otcextensions.sdk.elb.v2.listener.Listener`
            instance that the listener belongs to.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.elb.v2.listener_tag.Tag`,
            comprised of the properties on the Tag class.

        :returns: The results of the Tag Creation

        :rtype: :class:`~otcextensions.sdk.elb.v2.listener_tag.Tag`
        """
        listener_obj = self._get_resource(
            _listener.Listener,
            listener
        )
        return self._create(
            _lstnr_tag.Tag,
            listener_id=listener_obj.id,
            **attrs
        )

    def delete_listener_tag(self, listener, key, ignore_missing=True):
        """Delete a tag

        :param key: tag key
        :param listener: The listener can be either the ID of a
            listener or
            :class:`~otcextensions.sdk.elb.v2.listener.Listener`
            instance that the load_balancer belongs to..
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the tag does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent tag.

        :returns: ``None``
        """
        listener_obj = self._get_resource(
            _listener.Listener,
            listener
        )
        return self._delete(
            _lstnr_tag.Tag,
            key,
            listener_id=listener_obj.id,
            ignore_missing=ignore_missing
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.sdk.sfsturbo.v1 import _proxy
from otcextensions.sdk.sfsturbo.v1 import share as _share

from openstack.tests.unit import test_proxy_base


class TestSfstProxy(test_proxy_base.TestProxyBase):

    def setUp(self):
        super(TestSfstProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)

    def test_peering_create(self):
        self.verify_create(self.proxy.create_share, _share.Share,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_share_get(self):
        self.verify_get(self.proxy.get_share, _share.Share)

    def test_shares(self):
        self.verify_list(self.proxy.shares, _share.Share)

    def test_share_delete(self):
        self.verify_delete(
            self.proxy.delete_share, _share.Share, True,
            mock_method='otcextensions.sdk.sfsturbo.v1._proxy.Proxy._delete',
            expected_kwargs={
                'ignore_missing': True
            })

    def test_extend_capacity(self):
        share = _share.Share(id='share-id')
        self._verify(
            'otcextensions.sdk.sfsturbo.v1.share.Share.extend_capacity',
            self.proxy.extend_capacity,
            method_args=[share, 5],
            expected_args=[self.proxy],
            expected_kwargs={'extend': {'new_size': 5}}
        )

    def test_change_security_group(self):
        share = _share.Share(id='share-id')
        self._verify(
            'otcextensions.sdk.sfsturbo.v1.share.Share.change_security_group',
            self.proxy.change_security_group,
            method_args=[share, 'secgroup-uuid'],
            expected_args=[self.proxy],
            expected_kwargs={
                'change_security_group': {
                    'security_group_id': 'secgroup-uuid'}}
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base

from otcextensions.sdk.dms.v1 import product


JSON_DATA = {
    'tps': '50000',
    'storage': '600',
    'partition_num': '300',
    'product_id': '00300-30308-0--0',
    'spec_code': 'dms.instance.kafka.cluster.c3.mini',
    'io': [{
        'io_type': 'high',
        'storage_spec_code': 'dms.physical.storage.high',
        'available_zones': ['eu-de-02', 'eu-de-01'],
        'volume_type': 'SAS'
    }, {
        'io_type': 'ultra',
        'storage_spec_code': 'dms.physical.storage.ultra',
        'available_zones': ['eu-de-02', 'eu-de-01'],
        'volume_type': 'SSD'
    }],
    'bandwidth': '100MB',
    'unavailable_zones': ['eu-de-02'],
    'available_zones': ['eu-de-01'],
    'ecs_flavor_id': 'c4.large.2',
    'arch_type': 'X86'
}


class TestProduct(base.TestCase):

    def test_basic(self):
        sot = product.Product()

        self.assertEqual('/products', sot.base_path)

        self.assertTrue(sot.allow_list)

    def test_make_it(self):

        sot = product.Product(**JSON_DATA)
        self.assertEqual(JSON_DATA['tps'], sot.tps)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import random
import uuid

import mock

from openstackclient.tests.unit import utils

from otcextensions.tests.unit.osclient import test_base

from otcextensions.sdk.rds.v1.configuration import ConfigurationGroup
from otcextensions.sdk.rds.v1.datastore import Datastore
from otcextensions.sdk.rds.v1 import flavor
from otcextensions.sdk.rds.v1.instance import Instance
from otcextensions.sdk.rds.v1 import backup


class TestRds(utils.TestCommand):

    def setUp(self):
        super(TestRds, self).setUp()

        self.app.client_manager.rds = mock.Mock()

        self.client = self.app.client_manager.rds

        self.datastore_mock = FakeDatastore
        self.flavor_mock = FakeFlavor
        self.instance_mock = FakeInstance
        self.configuration_mock = FakeConfiguration


class FakeDatastore(object):
    """Fake one or more datastore versions."""

    @staticmethod
    def create_one(attrs=None, methods=None):
        """Create a fake datastore.

        :param Dictionary attrs:
            A dictionary with all attributes
        :param Dictionary methods:
            A dictionary with all methods
        :return:
            A FakeResource object, with id, name, metadata, and so on
        """
        attrs = attrs or {}
        methods = methods or {}

        # Set default attributes.
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'name': 'name-' + uuid.uuid4().hex,
            'datastore': 'datastore-' + uuid.uuid4().hex,
            'image': 'image-' + uuid.uuid4().hex,
            'packages': 'packages-' + uuid.uuid4().hex,
            'active': 1,
        }

        # Overwrite default attributes.
        # object_info.update(attrs)
        return Datastore(**object_info)

    @staticmethod
    def create_multiple(attrs=None, methods=None, count=2):
        """Create multiple fake servers.

        :param Dictionary attrs:
            A dictionary with all attributes
        :param Dictionary methods:
            A dictionary with all methods
        :param int count:
            The number of servers to fake
        :return:
            A list of FakeResource objects faking the servers
        """
        objects = []
        for i in range(0, count):
            objects.append(FakeDatastore.create_one(attrs, methods))

        return objects


class FakeFlavor(test_base.Fake):
    """Fake one or more VBS Policy"""

    @classmethod
    def generate(cls):
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'name': 'name-' + uuid.uuid4().hex,
            'spec_code': uuid.uuid4().hex,
            'ram': random.randint(1, 10280),
        }
        obj = flavor.Flavor.existing(**object_info)
        return obj


class FakeConfiguration(object):
    """Fake one or more Configuration."""

    @staticmethod
    def create_one(attrs=None, methods=None):
        """Create a fake Configuration.

        :param Dictionary attrs:
            A dictionary with all attributes
        :param Dictionary methods:
            A dictionary with all methods
        :return:
            A FakeResource object, with id, name, metadata, and so on
        """
        attrs = attrs or {}
        methods = methods or {}

        # Set default attributes.
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'name': 'name-' + uuid.uuid4().hex,
            'description': 'descriptions-' + uuid.uuid4().hex,
            'datastore_name': uuid.uuid4().hex,
            'datastore_version_name': uuid.uuid4().hex,
            'values': {},
        }

        # Overwrite default attributes.
        # object_info.update(attrs)
        return ConfigurationGroup(**object_info)

    @staticmethod
    def create_multiple(attrs=None, methods=None, count=2):
        """Create multiple fake Configuration.

        :param Dictionary attrs:
            A dictionary with all attributes
        :param Dictionary methods:
            A dictionary with all methods
        :param int count:
            The number of servers to fake
        :return:
            A list of FakeResource objects faking the servers
        """
        objects = []
        for i in range(0, count):
            objects.append(
                FakeConfiguration.create_one(attrs, methods)
            )

        return objects


class FakeInstance(object):
    """Fake one or more Instance."""

    @staticmethod
    def create_one(attrs=None, methods=None):
        """Create a fake Configuration.

        :param Dictionary attrs:
            A dictionary with all attributes
        :param Dictionary methods:
            A dictionary with all methods
        :return:
            A FakeResource object, with id, name, metadata, and so on
        """
        attrs = attrs or {}
        methods = methods or {}

        # Set default attributes.
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'name': 'name-' + uuid.uuid4().hex,
            'status': 'status-' + uuid.uuid4().hex,
            'datastore': {
                'type': 'datastore-' + uuid.uuid4().hex,
                'version': 'version-' + uuid.uuid4().hex,
            },
            'flavor': {'id': uuid.uuid4().hex},
            'volume': {
                'type': 'type' + uuid.uuid4().hex,
                'size': random.randint(1, 10280),
            },
            'region': 'region' + uuid.uuid4().hex,
        }

        # Overwrite default attributes.
        # object_info.update(attrs)
        return Instance(**object_info)

    @staticmethod
    def create_multiple(attrs=None, methods=None, count=2):
        """Create multiple fake Configuration.

        :param Dictionary attrs:
            A dictionary with all attributes
        :param Dictionary methods:
            A dictionary with all methods
        :param int count:
            The number of servers to fake
        :return:
            A list of FakeResource objects faking the servers
        """
        objects = []
        for i in range(0, count):
            objects.append(
                FakeInstance.create_one(attrs, methods)
            )

        return objects


class FakeBackup(test_base.Fake):
    """Fake one or more VBS Policy"""

    @classmethod
    def generate(cls):
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'name': 'name-' + uuid.uuid4().hex,
            'description': uuid.uuid4().hex,
            'datastore': {
                'type': 'datastore-' + uuid.uuid4().hex,
                'version': 'version-' + uuid.uuid4().hex,
            },
            'instance_id': 'instance_id-' + uuid.uuid4().hex,
            'size': random.randint(0, 100),
            'status': random.choice(['BUILDING', 'COMPLETED', 'FAILED',
                                     'DELETING']),
            'created': uuid.uuid4().hex,
            'updated': uuid.uuid4().hex,
        }
        obj = backup.Backup.existing(**object_info)
        return obj



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import test_proxy_base

from otcextensions.sdk.dds.v3 import _proxy
from otcextensions.sdk.dds.v3 import datastore as _datastore
from otcextensions.sdk.dds.v3 import instance as _instance
from otcextensions.sdk.dds.v3 import flavor as _flavor


class TestDdsProxy(test_proxy_base.TestProxyBase):

    def setUp(self):
        super(TestDdsProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestDatastore(TestDdsProxy):
    def test_datastore_types(self):
        res = list(self.proxy.datastore_types())
        self.assertEqual('DDS-Community', res[0].name)

    def test_datastores(self):
        self.verify_list(
            self.proxy.datastores,
            _datastore.Datastore,
            method_kwargs={
                'datastore_name': 'foo'},
            expected_kwargs={
                'datastore_name': 'foo'
            }
        )


class TestFlavor(TestDdsProxy):
    def test_flavors(self):
        self.verify_list(
            self.proxy.flavors,
            _flavor.Flavor,
            method_kwargs={
                'region': 'foo',
                'engine_name': 'engine',
            },
            expected_kwargs={
                'region': 'foo',
                'engine_name': 'engine',
            }
        )


class TestInstance(TestDdsProxy):
    def test_get_instance(self):
        self.verify_get(
            self.proxy.get_instance,
            _instance.Instance
        )

    def test_create_instance(self):
        self.verify_create(
            self.proxy.create_instance,
            _instance.Instance,)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class MicroserviceLabelSpec(resource.Resource):
    label_name = resource.Body("label_name")
    label_value = resource.Body("label_value")


class MemberGroupCreateSpec(resource.Resource):
    member_group_name = resource.Body("member_group_name")
    member_group_remark = resource.Body("member_group_remark")
    member_group_weight = resource.Body("member_group_weight", type=int)
    dict_code = resource.Body("dict_code")
    microservice_version = resource.Body("microservice_version")
    microservice_port = resource.Body("microservice_port", type=int)
    microservice_labels = resource.Body("microservice_labels", type=list,
                                        list_type=MicroserviceLabelSpec)
    member_group_id = resource.Body("member_group_id")
    create_time = resource.Body("create_time")
    update_time = resource.Body("update_time")


class MemberInfoSpec(resource.Resource):
    host = resource.Body("host")
    weight = resource.Body("weight", type=int)
    is_backup = resource.Body("is_backup", type=bool)
    member_group_name = resource.Body("member_group_name")
    status = resource.Body("status", type=int)
    port = resource.Body("port", type=int)
    ecs_id = resource.Body("ecs_id")
    ecs_name = resource.Body("ecs_name")


class VpcHealthConfigSpec(resource.Resource):
    protocol = resource.Body("protocol")
    path = resource.Body("path")
    method = resource.Body("method")
    port = resource.Body("port", type=int)
    threshold_normal = resource.Body("threshold_normal", type=int)
    threshold_abnormal = resource.Body("threshold_abnormal", type=int)
    time_interval = resource.Body("time_interval", type=int)
    http_code = resource.Body("http_code")
    enable_client_ssl = resource.Body("enable_client_ssl", type=bool)
    status = resource.Body("status", type=int)
    timeout = resource.Body("timeout", type=int)
    vpc_channel_id = resource.Body("vpc_channel_id")
    id = resource.Body("id")
    create_time = resource.Body("create_time")


class MicroServiceInfoCSEBaseSpec(resource.Resource):
    engine_id = resource.Body("engine_id")
    service_id = resource.Body("service_id")
    engine_name = resource.Body("engine_name")
    service_name = resource.Body("service_name")
    register_address = resource.Body("register_address")
    cse_app_id = resource.Body("cse_app_id")
    version = resource.Body("version")


class MicroServiceInfoCCEBaseSpec(resource.Resource):
    cluster_id = resource.Body("cluster_id")
    namespace = resource.Body("namespace")
    workload_type = resource.Body("workload_type")
    app_name = resource.Body("app_name")
    label_key = resource.Body("label_key")
    label_value = resource.Body("label_value")
    cluster_name = resource.Body("cluster_name")


class MicroServiceCreateSpec(resource.Resource):
    service_type = resource.Body("service_type")
    cse_info = resource.Body("cse_info",
                             type=MicroServiceInfoCSEBaseSpec)
    cce_info = resource.Body("cce_info",
                             type=MicroServiceInfoCCEBaseSpec)
    id = resource.Body("id")
    instance_id = resource.Body("instance_id")
    create_time = resource.Body("create_time")
    update_time = resource.Body("update_time")


class VpcChannel(resource.Resource):
    base_path = f'apigw/instances/%(gateway_id)s/vpc-channels'
    resources_key = 'vpc_channels'
    _query_mapping = resource.QueryParameters('limit', 'offset',
                                              'id', 'dict_code',
                                              'name', 'precise_search',
                                              'member_host', 'member_port',
                                              'member_group_name',
                                              'member_group_id')

    allow_create = True
    allow_delete = True
    allow_list = True
    allow_fetch = True
    allow_commit = True

    gateway_id = resource.URI('gateway_id')
    name = resource.Body('name')
    port = resource.Body('port', type=int)
    balance_strategy = resource.Body('balance_strategy', type=int)
    member_type = resource.Body('member_type')
    type = resource.Body('type', type=int)
    dict_code = resource.Body('dict_code')
    member_groups = resource.Body('member_groups', type=list,
                                  list_type=MemberGroupCreateSpec)
    members = resource.Body('members', type=list,
                            list_type=MemberInfoSpec)
    vpc_health_config = resource.Body('vpc_health_config',
                                      type=VpcHealthConfigSpec)
    microservice_info = resource.Body('microservice_info',
                                      type=MicroServiceCreateSpec)
    create_time = resource.Body('create_time')
    id = resource.Body('id')
    status = resource.Body('status', type=int)

    def modify_healthcheck(self, session, gateway_id, vpc_channel_id, **attrs):
        uri = (f'apigw/instances/{gateway_id}/vpc-channels/'
               f'{vpc_channel_id}/health-config')
        response = session.put(uri, json=attrs)
        exceptions.raise_from_response(response)
        data = response.json()
        self.vpc_health_config = VpcHealthConfigSpec.existing(**data)
        return self



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import uuid

from otcextensions.tests.functional.osclient.dcaas.v2 import common


class TestVirtualGateway(common.DcaasTestCase):

    UUID = uuid.uuid4().hex[:8]
    VG_NAME = "gateway-" + UUID

    def setUp(self):
        super(TestVirtualGateway, self).setUp()

    def tearDown(self):
        try:
            self._delete_virtual_gateway()
        finally:
            super(TestVirtualGateway, self).tearDown()

    def _create_virtual_gateway(self):
        json_output = json.loads(self.openstack(
            'dcaas gateway create '
            '{vpc_id} '
            '{local_ep_group_id} '
            '--name {name} -f json '.format(
                name=self.VG_NAME,
                vpc_id=self.VPC_ID,
                local_ep_group_id=self.EP_GROUP_ID
            )
        ))
        self.VG_ID = json_output['id']
        return json_output

    def _delete_virtual_gateway(self):
        self.openstack('dcaas gateway delete ' + self.VG_ID)

    def test_create_virtual_gateway(self):
        json_output = self._create_virtual_gateway()
        self.assertIsNotNone(json_output)
        self.assertEqual(json_output['id'], self.VG_ID)

    def test_list_virtual_gateway(self):
        self._create_virtual_gateway()
        json_output = json.loads(self.openstack(
            'dcaas gateway list -f json'
        ))
        self.assertIsNotNone(json_output)

    def test_list_filter_virtual_gateway(self):
        self._create_virtual_gateway()
        json_output = json.loads(self.openstack(
            'dcaas gateway list '
            '--name {name} '
            '--vpc_id {vpc_id} '
            '--local_ep_group_id {local_ep_group_id} -f json'.format(
                name=self.VG_NAME,
                vpc_id=self.VPC_ID,
                local_ep_group_id=self.EP_GROUP_ID
            )
        ))
        self.assertIsNotNone(json_output)

    def test_find_virtual_gateway(self):
        self._create_virtual_gateway()
        json_output = json.loads(self.openstack(
            'dcaas gateway show {id} -f json'.format(id=self.VG_ID)
        ))
        self.assertIsNotNone(json_output)

    def test_update_virtual_gateway(self):
        self._create_virtual_gateway()
        gateway_name = self.VG_NAME + '-updated'
        json_output = json.loads(self.openstack(
            'dcaas gateway update {id} --name {name} -f json'.format(
                id=self.VG_ID,
                name=gateway_name
            )
        ))
        self.assertEqual(json_output['id'], self.VG_ID)
        self.assertEqual(json_output['name'], gateway_name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack.tests.unit import base
from otcextensions.sdk.modelartsv1.v1 import training_job
from otcextensions.tests.unit.sdk.modelartsv1.v1 import examples
from otcextensions.tests.unit.sdk.utils import assert_attributes_equal

EXAMPLE = examples.TRAINING_JOB

EXAMPLE_CREATE = {
    "job_name": "TestModelArtsJob3",
    "workspace_id": "0",
    "config": {
        "worker_server_num": 1,
        "app_url": "/usr/app/",
        "boot_file_url": "/usr/app/boot.py",
        "parameter": [
            {"label": "learning_rate", "value": "0.01"},
            {"label": "batch_size", "value": "32"},
        ],
        "dataset_id": "dataset-id",
        "dataset_version_id": "version-id",
        "spec_id": 1,
        "engine_id": 1,
        "train_url": "/usr/train/",
        "log_url": "/usr/log/",
        "volumes": [
            {
                "nfs": {
                    "id": "nfs-id",
                    "src_path": "192.168.8.150:/",
                    "dest_path": "/home/work/nas",
                    "read_only": False,
                }
            },
            {
                "host_path": {
                    "src_path": "/root/work",
                    "dest_path": "/home/mind",
                    "read_only": False,
                }
            },
        ],
    },
}


class TestTrainingjob(base.TestCase):
    def setUp(self):
        super(TestTrainingjob, self).setUp()

    def test_basic(self):
        sot = training_job.TrainingJob()

        self.assertEqual("/training-jobs", sot.base_path)
        self.assertEqual(None, sot.resource_key)
        self.assertEqual("jobs", sot.resources_key)

        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_fetch)

    def test_make_it(self):
        sot = training_job.TrainingJob(**EXAMPLE)
        self.assertEqual(sot.id, EXAMPLE["job_id"])
        for key, value in EXAMPLE.items():
            if key == "create_time":
                self.assertEqual(sot.created_at, EXAMPLE[key])
            elif key == "job_desc":
                self.assertEqual(sot.job_description, EXAMPLE[key])
            else:
                assert_attributes_equal(self, getattr(sot, key), value)

    def test_create_sot(self):
        sot = training_job.TrainingJob(**EXAMPLE_CREATE)
        assert_attributes_equal(self, sot, EXAMPLE_CREATE)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import logging

from osc_lib import utils

from otcextensions import sdk
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)

DEFAULT_API_VERSION = '1'
API_VERSION_OPTION = 'os_dws_api_version'
API_NAME = "dws"
API_VERSIONS = {
    "1": "openstack.connection.Connection",
}


def make_client(instance):
    """Returns a dws proxy"""

    conn = instance.sdk_connection

    if getattr(conn, 'dws', None) is None:
        LOG.debug('OTC extensions are not registered. Do that now')
        sdk.register_otc_extensions(conn)

    LOG.debug('DWS client initialized using OpenStack OTC SDK: %s',
              conn.dws)
    return conn.dws


def build_option_parser(parser):
    """Hook to add global options"""
    parser.add_argument(
        '--os-dws-api-version',
        metavar='<dws-api-version>',
        default=utils.env('OS_DWS_API_VERSION'),
        help=_("DWS API version, default=%s "
               "(Env: OS_DWS_API_VERSION)") % DEFAULT_API_VERSION
    )
    return parser



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.identity.v3 import credential


FAKE_ID = "945d-fe449be00148"
EXAMPLE = {
    "access": FAKE_ID,
    "create_time": "2020-01-08T06:26:08.123059Z",
    "user_id": "07609fb9358010e21f7bc0037...",
    "description": "",
    "status": "active"
}


class TestCredential(base.TestCase):

    def setUp(self):
        super(TestCredential, self).setUp()

    def test_basic(self):
        sot = credential.Credential()

        self.assertEqual('/v3.0/OS-CREDENTIAL/credentials', sot.base_path)
        self.assertEqual('credentials', sot.resources_key)
        self.assertEqual('credential', sot.resource_key)
        self.assertEqual('PUT', sot.commit_method)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

        self.assertDictEqual({
            'limit': 'limit',
            'marker': 'marker',
            'user_id': 'user_id'},
            sot._query_mapping._mapping
        )

    def test_make_it(self):

        sot = credential.Credential(connection=self.cloud, **EXAMPLE)
        # Check how the override with "real" connection works
        self.assertEqual(
            'https://identity.example.com/v3.0/OS-CREDENTIAL/credentials',
            sot.base_path)

        self.assertEqual(EXAMPLE['access'], sot.id)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['create_time'], sot.created_at)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class ImageInfoListSpec(resource.Resource):
    #: Image engine type.
    datastore_type = resource.Body('datastoreType')
    #: Image engine version.
    datastore_version = resource.Body('datastoreVersion')
    #: Name of an image that can be upgraded.
    display_name = resource.Body('displayName')
    #: ID of an image that can be upgraded.
    id = resource.Body('id')
    #: Image description.
    image_desc = resource.Body('imageDesc')
    #: Priority.
    priority = resource.Body('priority', type=int)


class ClusterImage(resource.Resource):
    base_path = '/clusters/%(cluster_id)s/target/%(upgrade_type)s/images'
    allow_fetch = True
    #: Cluster ID.
    cluster_id = resource.URI('cluster_id')
    #: Version type. The value can be:
    #: \nsame: upgrade to the same version.
    #: \ncross: upgrade to a different version.
    #: \ncrossEngine: cross-engine upgrade.
    upgrade_type = resource.URI('upgrade_type')
    #: Indicates whether to upload the plug-in of the target version.
    need_upload_upgrade_plugin = resource.Body(
        'needUploadUpgradePlugin', type=bool
    )
    #: Image details.
    image_info_list = resource.Body(
        'imageInfoList', type=list, list_type=ImageInfoListSpec
    )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Identity custom role v3.0 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListCustomRoles(command.Lister):
    _description = _('List Identity Custom Roles')
    columns = (
        'id',
        'name',
        'description',
        'domain_id',
        'references',
        'catalog',
        'display_name',
        'type',
        'created_at',
        'updated_at',
    )

    def get_parser(self, prog_name):
        parser = super(ListCustomRoles, self).get_parser(prog_name)

        parser.add_argument(
            '--page',
            metavar='<page>',
            help=_('Page number for pagination query.')
        )
        parser.add_argument(
            '--per_page',
            metavar='<per_page>',
            help=_('Number of data records to be displayed on each page.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.iam

        table_columns = (
            'ID',
            'Name',
            'Description',
            'Domain ID',
            'References',
            'Catalog',
            'Display name',
            'Type',
            'Created At',
            'Updated At',
        )

        attrs = {}

        if parsed_args.page:
            attrs['page'] = parsed_args.page
        if parsed_args.page:
            attrs['per_page'] = parsed_args.per_page
        data = client.custom_roles(**attrs)

        table = (table_columns,
                 (utils.get_dict_properties(
                     s, self.columns
                 ) for s in data))
        return table



# Licensed under the Apache License, Version 2.0 (the "License"); you may

# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import xml.etree.ElementTree as ET

from openstack import _log
from openstack import exceptions
from openstack import resource
from openstack import utils

from otcextensions.sdk.obs.v1 import _base


_logger = _log.setup_logging('openstack')


class Container(_base.BaseResource):

    resources_key = 'Buckets'
    resource_key = 'Bucket'

    allow_get = True
    allow_head = True
    allow_list = True
    allow_create = True
    allow_delete = True

    create_method = 'PUT'

    base_path = '/'

    # all requests (except create) will default to requires_id = None
    requires_id = None

    name = resource.Body('Name', alternate_id=True, alias='id')
    creation_date = resource.Body('CreationDate')

    # When creating a bucket, you can use this parameter
    # to set a pre-defined ACL.
    storage_acl = resource.Header('x-amz-acl')
    # When creating a bucket, you can add this header
    # to set the default storage class for the bucket.
    # Value range:
    # STANDARD (Standard storage)
    # WARM (Warm storage)
    # COLD (Cold storage)
    storage_class = resource.Header('x-obs-storage-class')
    # Grants the read permission to all users in a specified domain.
    grant_read = resource.Header('x-obs-grant-read')
    # Grants the WRITE permission to all users in a specified domain to create,
    # delete, and overwrite all objects in a bucket; and initiate multipart
    # uploads, upload parts, copy parts, assemble parts,
    # and cancel multipart uploads.
    grant_write = resource.Header('x-obs-grant-write')
    # Grant the READ_ACP permission to all users in a specified domain
    # to allow them to read the bucket ACL.
    grant_read_acp = resource.Header('x-obs-grant-read-acp')
    # Grants the WRITE_ACP permission to all users in a specified domain
    # to allow them to modify the bucket ACL.
    grant_write_acp = resource.Header('x-obs-grant-write-acp')
    # Grants the FULL_CONTROL permission to all users in a specified domain.
    grant_full_control = resource.Header('x-obs-grant-full-control')
    # Grants the READ permission to all users in a specified domain.
    # By default, the read permission is granted on all objects in the bucket.
    grant_read_delivered = resource.Header('x-obs-grant-read-delivered')
    # Grants the FULL_CONTROL permission to all users in a specified domain.
    # By default, the FULL_CONTROL permission is granted on all
    # objects in the bucket.
    grant_full_delivered = resource.Header(
        'x-obs-grant-full-control-delivered'
    )
    # This header can be carried when you want to create
    # a parallel file system.
    fs_file_interface = resource.Header('x-obs-fs-file-interface')
    # When creating a bucket,
    # you can use this header to enable WORM for the bucket.
    object_lock_enabled = resource.Header('x-obs-bucket-object-lock-enabled')

    def _translate_response(self, response, has_body=True, error_message=None,
                            resource_response_key=None):
        """Given a KSA response, inflate this instance with its data

        This method updates attributes that correspond to headers
        and body on this instance and clears the dirty set.
        """
        exceptions.raise_from_response(response, error_message=response.text)
        if response:
            if has_body:
                # TODO(agoncharov): do nothing so far. Generally need
                # to parse different responses
                pass

    def _prepare_request(self, requires_id=None, prepend_key=False):
        """Prepare a request to be sent to the server

        Create operations don't require an ID, but all others do,
        so only try to append an ID when it's needed with
        requires_id. Create and update operations sometimes require
        their bodies to be contained within an dict -- if the
        instance contains a resource_key and prepend_key=True,
        the body will be wrapped in a dict with that key.

        Return a _Request object that contains the constructed URI
        as well a body and headers that are ready to send.
        Only dirty body and header contents will be returned.
        """
        if requires_id is None:
            requires_id = self.requires_id

        body = None
        # body = self._body.dirty
        # if prepend_key and self.resource_key is not None:
        #     body = {self.resource_key: body}

        # if self.name:
        #     body['Bucket'] = self.name

        base_path = '/'
        headers = {}
        for k, v in self._header.dirty.items():
            if isinstance(v, list):
                headers[k] = ", ".join(v)
            else:
                headers[k] = str(v)
        uri = base_path % self._uri.attributes
        if requires_id:
            if self.id is None:
                raise exceptions.InvalidRequest(
                    "Request requires an ID but none was found")

            uri = utils.urljoin(uri, self.id)

        return resource._Request(uri, body, headers)

    @classmethod
    def list(cls, session, paginated=False,
             endpoint_override=None, headers=None, requests_auth=None,
             **params):
        if not cls.allow_list:
            raise exceptions.MethodNotSupported(cls, "list")

        cls._query_mapping._validate(params, base_path=cls.base_path)
        query_params = cls._query_mapping._transpose(params, cls)

        response = session.get(
            session.get_endpoint(),
            params=query_params.copy(),
            requests_auth=requests_auth
        )

        root = ET.fromstring(response.content)

        if root.tag != ET.QName(cls.OBS_NS, 'ListAllMyBucketsResult'):
            _logger.warn('Namespace in the response does not match '
                         'expectation')
            cls.OBS_NS = root.tag.split('}', 1)[0][1:]

        for elements in root:

            if elements.tag == ET.QName(cls.OBS_NS, cls.resources_key):
                for el in elements:
                    if el.tag == ET.QName(cls.OBS_NS, cls.resource_key):
                        # Convert XML part into dict
                        dict_raw_resource = cls.etree_to_dict(el)
                        # extract resource data
                        dict_resource = dict_raw_resource[cls.resource_key]
                        value = cls.existing(**dict_resource)
                        yield value

        return

    def create(self, session, prepend_key=True,
               endpoint_override=None, headers=None, requests_auth=None,
               **attrs):

        if not self.allow_create:
            raise exceptions.MethodNotSupported(self, "create")

        session = self._get_session(session)

        request = self._prepare_request()

        req_args = self._prepare_override_args(
            endpoint_override=endpoint_override,
            request_headers=request.headers,
            additional_headers=headers,
            requests_auth=requests_auth)
        # hack for nl, strange that optional parameter
        # LocationConstraint is required there
        if requests_auth.aws_region == 'eu-nl':
            request.body = '''<CreateBucketConfiguration>
                <LocationConstraint>{region}</LocationConstraint>
                </CreateBucketConfiguration>
            '''.format(region=requests_auth.aws_region)
        response = session.put(request.url,
                               data=request.body, **req_args)

        self._translate_response(response)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import time

from keystoneauth1 import session
from keystoneauth1.tests.unit import utils

from otcextensions.common import agency_auth


class AgencyTest(utils.TestCase):

    TEST_ROOT_URL = 'http://127.0.0.1:5000/'
    TEST_URL = '%s%s' % (TEST_ROOT_URL, 'v3')
    TEST_ROOT_ADMIN_URL = 'http://127.0.0.1:35357/'
    TEST_ADMIN_URL = '%s%s' % (TEST_ROOT_ADMIN_URL, 'v3')

    TEST_PASS = 'password'

    TEST_APP_CRED_ID = 'appcredid'
    TEST_APP_CRED_SECRET = 'secret'
    TEST_TARGET_DOMAIN_ID = 'agency_domain_id'
    TEST_TARGET_AGENCY_NAME = 'agency_name'
    TEST_TARGET_PROJECT_ID = 'target_project_id'

    def setUp(self):
        super(AgencyTest, self).setUp()
        nextyear = 1 + time.gmtime().tm_year
        self.TEST_RESPONSE_DICT = {
            "token": {
                "methods": [
                    "token",
                    "password"
                ],

                "expires_at": "%i-02-01T00:00:10.000123Z" % nextyear,
                "project": {
                    "domain": {
                        "id": self.TEST_DOMAIN_ID,
                        "name": self.TEST_DOMAIN_NAME
                    },
                    "id": self.TEST_TENANT_ID,
                    "name": self.TEST_TENANT_NAME
                },
                "user": {
                    "domain": {
                        "id": self.TEST_DOMAIN_ID,
                        "name": self.TEST_DOMAIN_NAME
                    },
                    "id": self.TEST_USER,
                    "name": self.TEST_USER
                },
                "issued_at": "2013-05-29T16:55:21.468960Z",
                "catalog": [],  # self.TEST_SERVICE_CATALOG,
                "service_providers": []  # self.TEST_SERVICE_PROVIDERS
            },
        }

    def stub_auth(self, subject_token=None, **kwargs):
        if not subject_token:
            subject_token = self.TEST_TOKEN

        self.stub_url('POST', ['auth', 'tokens'],
                      headers={'X-Subject-Token': subject_token}, **kwargs)

    def test_authenticate_with_username_password(self):
        self.stub_auth(json=self.TEST_RESPONSE_DICT)
        a = agency_auth.Agency(self.TEST_URL,
                               username=self.TEST_USER,
                               password=self.TEST_PASS,
                               target_domain_id=self.TEST_TARGET_DOMAIN_ID,
                               target_agency_name=self.TEST_TARGET_AGENCY_NAME,
                               target_project_id=self.TEST_TARGET_PROJECT_ID)
        self.assertFalse(a.has_scope_parameters)
        s = session.Session(auth=a)

        self.assertEqual({'X-Auth-Token': self.TEST_TOKEN},
                         s.get_auth_headers())

        req = {'auth': {'identity':
               {'methods': ['assume_role'],
                'assume_role': {'domain_id': self.TEST_TARGET_DOMAIN_ID,
                                'xrole_name': self.TEST_TARGET_AGENCY_NAME}},
               'scope': {'project': {'id': self.TEST_TARGET_PROJECT_ID}}}}

        self.assertRequestBodyIs(json=req)
        self.assertRequestHeaderEqual('Content-Type', 'application/json')
        self.assertRequestHeaderEqual('Accept', 'application/json')
        self.assertRequestHeaderEqual('X-Auth-Token', self.TEST_TOKEN)
        self.assertEqual(s.auth.auth_ref.auth_token, self.TEST_TOKEN)
        # TODO(not_gtema): add other tests

    def test_authenticate_with_username_password_roles(self):
        self.stub_auth(json=self.TEST_RESPONSE_DICT)
        a = agency_auth.Agency(self.TEST_URL,
                               username=self.TEST_USER,
                               password=self.TEST_PASS,
                               target_domain_id=self.TEST_TARGET_DOMAIN_ID,
                               target_agency_name=self.TEST_TARGET_AGENCY_NAME,
                               target_project_id=self.TEST_TARGET_PROJECT_ID,
                               roles=["r1", "r2"])
        self.assertFalse(a.has_scope_parameters)
        s = session.Session(auth=a)

        self.assertEqual({'X-Auth-Token': self.TEST_TOKEN},
                         s.get_auth_headers())

        req = {'auth': {'identity':
               {'methods': ['assume_role'],
                'assume_role': {'domain_id': self.TEST_TARGET_DOMAIN_ID,
                                'xrole_name': self.TEST_TARGET_AGENCY_NAME,
                                'restrict': {'roles': ['r1', 'r2']}}},
               'scope': {'project': {'id': self.TEST_TARGET_PROJECT_ID}}}}

        self.assertRequestBodyIs(json=req)
        self.assertRequestHeaderEqual('Content-Type', 'application/json')
        self.assertRequestHeaderEqual('Accept', 'application/json')
        self.assertRequestHeaderEqual('X-Auth-Token', self.TEST_TOKEN)
        self.assertEqual(s.auth.auth_ref.auth_token, self.TEST_TOKEN)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource

from otcextensions.common import format


class TaskStatus(resource.Resource):

    base_path = '/query_task_status'

    # capabilities
    allow_fetch = True

    _query_mapping = resource.QueryParameters('task_id')

    # Properties
    #: Status of task
    #: validate status are `success`, `failed`, `waiting`, `running`
    task_status = resource.Body('task_status')
    #: Additional task status message
    task_msg = resource.Body('task_msg')


class FloatingIPStatus(resource.Resource):

    base_path = '/antiddos/%(floating_ip_id)s/status'

    # capabilities
    allow_fetch = True

    # Properties
    floating_ip_id = resource.URI('floating_ip_id')
    #: Status of Anti-DDos
    #: validate status are `normal`, `configging`, `notConfig`,
    #: `packetcleaning`, `packetdropping`
    status = resource.Body('status')


class FloatingIPEvent(resource.Resource):

    resources_key = 'logs'
    base_path = '/antiddos/%(floating_ip_id)s/logs'

    _query_mapping = resource.QueryParameters('limit', 'offset', 'sort_dir')

    # capabilities
    allow_list = True

    # Properties
    floating_ip_id = resource.URI('floating_ip_id')
    #: start time
    #: *Type: int*
    start_time = resource.Body('start_time', type=format.TimeTMsStr)
    #: end time
    #: *Type: int*
    end_time = resource.Body('end_time', type=format.TimeTMsStr)
    #: Anti-ddos status
    #: Defense status, the possible value of which is one of the following:
    #: * 1: indicates that traffic cleaning is underway.
    #: * 2: indicates that traffic is discarded.
    #: *Type: int*
    status = resource.Body('status', type=int)
    #: Trigger bps (bit/s)
    #: *Type: int*
    trigger_bps = resource.Body('trigger_bps', type=int)
    #: Trigger package per second
    #: *Type: int*
    trigger_pps = resource.Body('trigger_pps', type=int)
    #: Trigger http requests
    #: *Type: int*
    trigger_http_pps = resource.Body('trigger_http_pps', type=int)


class FloatingIPDayStat(resource.Resource):

    resources_key = 'data'
    base_path = '/antiddos/%(floating_ip_id)s/daily'

    # capabilities
    allow_list = True

    # Properties
    #: Data start time
    #: *Type: int*
    period_start = resource.Body('period_start', type=format.TimeTMsStr)
    #: In (bit/s)
    #: *Type: int*
    bps_in = resource.Body('bps_in', type=int)
    #: Attack (bit/s)
    #: *Type: int*
    bps_attack = resource.Body('bps_attack', type=int)
    #: Total data (bit/s)
    #: *Type: int*
    total_bps = resource.Body('total_bps', type=int)
    #: Package in speed (/s)
    #: *Type: int*
    pps_in = resource.Body('pps_in', type=int)
    #: Package attack speed (/s)
    #: *Type: int*
    pps_attack = resource.Body('pps_attack', type=int)
    #: Total package speed (/s)
    #: *Type: int*
    total_pps = resource.Body('total_pps', type=int)


class FloatingIPWeekStatData(resource.Resource):
    # Properties
    #: Intercept time in one week
    #: *Type: int*
    ddos_intercept_times = resource.Body('ddos_intercept_times', type=int)
    #: *Type: int*
    ddos_blackhole_times = resource.Body('ddos_blackhole_times', type=int)
    #: *Type: int*
    max_attack_bps = resource.Body('max_attack_bps', type=int)
    #: *Type: int*
    max_attack_conns = resource.Body('max_attack_conns ', type=int)
    #: Data start time
    #: *Type: int*
    period_start_date = resource.Body('period_start_date',
                                      type=format.TimeTMsStr)


class FloatingIPWeekStat(resource.Resource):

    base_path = '/antiddos/weekly'

    # capabilities
    allow_fetch = True

    _query_mapping = resource.QueryParameters('period_start_date')

    # Properties
    #: Intercept time in one week
    #: *Type: int*
    ddos_intercept_times = resource.Body('ddos_intercept_times', type=int)
    #: A list of data in one week
    #: *Type: list*
    weekdata = resource.Body('weekdata', type=list,
                             list_type=FloatingIPWeekStatData)
    #: Top 10 ip address in one week
    #: *Type: list*
    top10 = resource.Body('top10', type=list)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import _log
from openstack import resource

from otcextensions.sdk import sdk_resource

_logger = _log.setup_logging('openstack')


class Backup(sdk_resource.Resource):

    base_path = '/backups'
    resource_key = 'backup'
    resources_key = 'backups'
    service_expectes_json_type = True

    # capabilities
    allow_create = True
    allow_delete = True
    allow_list = True

    # Properties
    # project_id = resource.URI('project_id')
    #: Backup id
    #: Type: uuid*
    id = resource.Body('id')
    #: Instance id
    instance_id = resource.Body('instance_id')
    #: Instance id alt
    instance = resource.Body('instance')
    #: Backup created time
    created = resource.Body('created')
    #: Data store information
    #: *Type: dict*
    datastore = resource.Body('dataStore', type=dict)
    #: Data backup description
    description = resource.Body('description')
    #: Back file name
    name = resource.Body('name')
    #: Back file size in GB
    #: *Type:int*
    size = resource.Body('size', type=int)
    #: Backup status
    status = resource.Body('status')
    #: Finished time
    updated = resource.Body('updated')
    #: Backup type
    backuptype = resource.Body('backuptype')

    # @classmethod
    # def new(cls, **attrs):
    #     return Backup(
    #         content_type='application/json',
    #         x_language='en-us',
    #         **attrs)


class BackupPolicy(sdk_resource.Resource):

    base_path = '/instances/%(instance_id)s/backups/policy'
    resource_key = 'policy'

    # capabilities
    allow_update = True
    allow_get = True

    #: instaceId
    instance_id = resource.URI('instance_id')
    # project_id = resource.URI('project_id')

    # Properties
    #: Policy keep days
    #:  Indicates the number of days to retain the generated backup files.
    #:  Its value range is 0 to 35. If this parameter is 0,
    #:  the automated backup policy is not set.
    #: *Type: int*
    keepday = resource.Body('keepday', type=int)
    #: Start time
    #:  Indicates the backup start time that has been set.
    #:  The backup task will be triggered within one hour
    #:  after the backup start time.
    #:  The current time is the UTC time.
    #: *Type: string*
    starttime = resource.Body('starttime')

    # @classmethod
    # def new(cls, **attrs):
    #     return BackupPolicy(
    #         content_type='application/json',
    #         x_language='en-us', **attrs)

    # use put to create, but we don't require id
    def update(self, session, prepend_key=True,
               endpoint_override=None, headers=None):
        """Create a remote resource based on this instance.

        Method is overriden, because PUT without ID should be used

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param prepend_key: A boolean indicating whether the resource_key
                            should be prepended in a resource creation
                            request. Default to True.

        :return: None.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_create` is not set to ``True``.
        """
        self.update_no_id(
            session, prepend_key,
            # endpoint_override=endpoint_override,
            headers=headers)

        return None



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import openstack

from otcextensions.tests.functional.sdk.dcaas.v2 import base

_logger = openstack._log.setup_logging('openstack')


class TestDirectConnectEndpointGroup(base.BaseDCTest):

    UUID = base.BaseDCTest.UUID
    CIDR = base.BaseDCTest.CIDR
    TYPE = "cidr"
    EG_NAME = "test-eg-" + UUID

    def setUp(self):
        super(TestDirectConnectEndpointGroup, self).setUp()
        self.dcaas = self.conn.dcaas
        self._create_endpoint_group()

    def tearDown(self):
        try:
            self._delete_endpoint_group()
        finally:
            super(TestDirectConnectEndpointGroup, self).tearDown()

    def _create_endpoint_group(self):
        project_id = self.conn.session.get_project_id()
        endpoints = [self.CIDR]
        self.endpoint_group = self.dcaas.create_endpoint_group(
            project_id=project_id,
            name=self.EG_NAME,
            endpoints=endpoints,
            type=self.TYPE
        )

    def _delete_endpoint_group(self):
        endpoint_groups = list(self.dcaas.endpoint_groups())
        for endpoint_group in endpoint_groups:
            self.dcaas.delete_endpoint_group(endpoint_group.id)

    def test_create_and_get_endpoint_group(self):
        endpoint_group = self.dcaas.get_endpoint_group(self.endpoint_group.id)
        self.assertEqual(endpoint_group.name, self.EG_NAME)

    def test_list_endpoint_group(self):
        endpoint_groups = list(self.dcaas.endpoint_groups())
        self.assertGreaterEqual(len(endpoint_groups), 1)

    def test_find_and_update_endpoint_group(self):
        updated_name = "updated_name"
        endpoint_group = self.dcaas.find_endpoint_group(self.EG_NAME)
        updated_ep_group = self.dcaas.update_endpoint_group(
            endpoint_group=endpoint_group,
            name=updated_name
        )
        self.assertEqual(self.endpoint_group.id, endpoint_group.id)
        self.assertEqual(endpoint_group.id, updated_ep_group.id)
        self.assertEqual(updated_ep_group.name, updated_name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import uuid

from openstack.tests.unit import base
from otcextensions.sdk.vpcep.v1 import endpoint
from otcextensions.tests.unit.sdk.utils import assert_attributes_equal

EXAMPLE = {
    'active_status': ['active'],
    'created_at': '2024-02-26T14:10:12Z',
    'description': '',
    'enable_dns': False,
    'enable_status': 'enable',
    'enable_whitelist': False,
    'endpoint_pool_id': uuid.uuid4().hex,
    'endpoint_service_id': uuid.uuid4().hex,
    'endpoint_service_name': 'eu-de.css-762-az1.' + uuid.uuid4().hex,
    'id': uuid.uuid4().hex,
    'ip': '192.168.0.232',
    'marker_id': 16952019,
    'project_id': uuid.uuid4().hex,
    'routetables': [],
    'service_type': 'interface',
    'specification_name': 'default',
    'status': 'accepted',
    'subnet_id': uuid.uuid4().hex,
    'tags': [
        {
            'key': 'test1',
            'value': 'test1',
        }
    ],
    'updated_at': '2024-02-26T14:10:14Z',
    'vpc_id': uuid.uuid4().hex,
    'whitelist': [],
}


class TestEndpoint(base.TestCase):

    def test_basic(self):
        sot = endpoint.Endpoint()
        self.assertEqual('endpoints', sot.resources_key)
        self.assertEqual(None, sot.resource_key)
        self.assertEqual('/vpc-endpoints', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

        self.assertDictEqual(
            {
                'id': 'id',
                'limit': 'limit',
                'router_id': 'vpc_id',
                'marker': 'marker',
                'offset': 'offset',
                'endpoint_service_name': 'endpoint_service_name',
                'sort_key': 'sort_key',
                'sort_dir': 'sort_dir',
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        updated_sot_attrs = {
            'enable_dns': 'is_dns_enabled',
            'enable_whitelist': 'is_whitelist_enabled',
            'routetables': 'route_tables',
            'vpc_id': 'router_id',
            'subnet_id': 'network_id',
            'enable_status': 'is_enabled',
        }
        if 'enable_status' in EXAMPLE.keys():
            enable_status = EXAMPLE['enable_status']
            EXAMPLE['enable_status'] = (
                True if enable_status == 'enable' else False
            )

        sot = endpoint.Endpoint(**EXAMPLE)
        for key, value in EXAMPLE.items():
            if key in updated_sot_attrs.keys():
                self.assertEqual(getattr(sot, updated_sot_attrs[key]), value)
            else:
                assert_attributes_equal(self, getattr(sot, key), value)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""Datastore v1 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _


LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListDatastores(command.Lister):

    _description = _("List available datastores")
    columns = ['Name']

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        data = client.datastore_types()

        return (
            self.columns,
            (utils.get_item_properties(
                s,
                self.columns,
            ) for s in data)
        )


class ShowDatastore(command.ShowOne):

    _description = _("Shows details of a datastore")
    columns = ['Name']

    def get_parser(self, prog_name):
        parser = super(ShowDatastore, self).get_parser(prog_name)
        parser.add_argument(
            'datastore',
            metavar='<datastore>',
            help=_('ID of the datastore'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        data = client.datastore_types()

        datastore = None
        for ds in data:
            if ds.name == parsed_args.datastore:
                datastore = ds

        return (
            self.columns,
            utils.get_item_properties(datastore, self.columns)
        )


class ListDatastoreVersions(command.Lister):
    _description = _("Lists available versions for a datastore")
    columns = ['ID', 'Name']

    def get_parser(self, prog_name):
        parser = super(ListDatastoreVersions, self).get_parser(prog_name)
        parser.add_argument(
            'datastore',
            metavar='<datastore>',
            help=_('Name of the datastore'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        data = client.datastore_versions(datastore=parsed_args.datastore)

        return (
            self.columns,
            (utils.get_item_properties(
                s,
                self.columns,
            ) for s in data)
        )


class ShowDatastoreVersion(command.ShowOne):
    _description = _("Shows details of a datastore version.")
    columns = ['Active', 'Datastore', 'ID', 'Image', 'Name', 'Packages', ]

    def get_parser(self, prog_name):
        parser = super(ShowDatastoreVersion, self).get_parser(prog_name)
        parser.add_argument(
            'datastore_version',
            metavar='<datastore_version>',
            help=_('ID or name of the datastore version.'),
        )
        parser.add_argument(
            '--datastore',
            metavar='<datastore>',
            default=None,
            required=True,
            help=_('ID or name of the datastore. Optional if the ID of'
                   'the datastore_version is provided.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        obj = client.get_datastore_version(
            datastore=parsed_args.datastore,
            datastore_version=parsed_args.datastore_version
        )

        # display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, self.columns)

        return (self.columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base
from otcextensions.sdk.dis.v2 import app


EXAMPLE = {
    "app_id": "bd6IPpvgiIflQPMpi9M",
    "app_name": "newapp",
    "create_time": 1593569685875,
    "commit_checkpoint_stream_names": ["newstream"]
}


class TestApp(base.TestCase):

    def test_basic(self):
        sot = app.App()
        self.assertEqual(None, sot.resource_key)
        self.assertEqual('apps', sot.resources_key)
        path = '/apps'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_patch)

    def test_make_it(self):
        sot = app.App(**EXAMPLE)
        updated_sot_attrs = (
            'app_name',
            'create_time',
        )
        self.assertEqual(EXAMPLE['create_time'], sot.created_at)
        self.assertEqual(EXAMPLE['app_id'], sot.id)
        self.assertEqual(EXAMPLE['app_name'], sot.name)
        for key, value in EXAMPLE.items():
            if key not in updated_sot_attrs:
                self.assertEqual(getattr(sot, key), value)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts Dataset Synchronization v2 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import cli_utils
from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)

_formatters = {
    "created_at": cli_utils.UnixTimestampFormatter,
}


def _get_columns(item):
    column_map = {}
    hidden = ["location"]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


class DatasetSync(command.Command):
    _description = _("Creating a Dataset Synchronization Task")

    def get_parser(self, prog_name):
        parser = super(DatasetSync, self).get_parser(prog_name)
        parser.add_argument(
            "dataset",
            metavar="<dataset>",
            help=_("Dataset Id or name."),
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv2
        dataset = client.find_dataset(
            parsed_args.dataset, ignore_missing=False
        )
        client.dataset_sync(dataset)


class DatasetSyncStatus(command.ShowOne):
    _description = _("Show details of a Modelarts Dataset.")

    def get_parser(self, prog_name):
        parser = super(DatasetSyncStatus, self).get_parser(prog_name)
        parser.add_argument(
            "dataset",
            metavar="<dataset>",
            help=_("Dataset Id or name."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv2

        dataset = client.find_dataset(
            parsed_args.dataset, ignore_missing=False
        )
        data = client.get_dataset_sync_status(dataset)

        display_columns, columns = _get_columns(data)
        data = utils.get_item_properties(data, columns, formatters=_formatters)

        return display_columns, data



# Licensed under the Apache License, Version 2.0 (the "License"); you may

# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class ImageExport(resource.Resource):
    base_path = '/cloudimages/%(image_id)s/file'

    allow_create = True
    requires_id = False

    image_id = resource.URI('image_id')
    bucket_url = resource.Body('bucket_url')
    file_format = resource.Body('file_format')
    is_quick_export = resource.Body('is_quick_export', type=bool)
    job_id = resource.Body('job_id')

    def create(self, session, prepend_key=False, base_path=None):
        # Overriden here to override prepend_key default value
        return super(ImageExport, self).create(session, prepend_key, base_path)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import uuid

from openstack.tests.unit import base
from otcextensions.sdk.vpcep.v1 import service
from otcextensions.tests.unit.sdk.utils import assert_attributes_equal

EXAMPLE = {
    'id': uuid.uuid4().hex,
    'port_id': uuid.uuid4().hex,
    'vpc_id': uuid.uuid4().hex,
    'pool_id': uuid.uuid4().hex,
    'status': 'available',
    'approval_enabled': False,
    'service_name': 'test123',
    'service_type': 'interface',
    'server_type': 'VM',
    'project_id': uuid.uuid4().hex,
    'created_at': '2018-01-30T07:42:01.174',
    'ports': [
        {'client_port': 8080, 'server_port': 90, 'protocol': 'TCP'},
        {'client_port': 8081, 'server_port': 80, 'protocol': 'TCP'},
    ],
}


class TestEndpointService(base.TestCase):

    def setUp(self):
        super(TestEndpointService, self).setUp()

    def test_basic(self):
        sot = service.Service()
        self.assertEqual('endpoint_services', sot.resources_key)
        self.assertEqual(None, sot.resource_key)
        self.assertEqual('/vpc-endpoint-services', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

        self.assertDictEqual(
            {
                'id': 'id',
                'limit': 'limit',
                'marker': 'marker',
                'name': 'endpoint_service_name',
                'offset': 'offset',
                'sort_dir': 'sort_dir',
                'sort_key': 'sort_key',
                'status': 'status',
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = service.Service(**EXAMPLE)
        updated_sot_attrs = {
            'approval_enabled': 'is_approval_enabled',
            'vpc_id': 'router_id',
        }
        for key, value in EXAMPLE.items():
            if key in updated_sot_attrs.keys():
                self.assertEqual(
                    getattr(sot, updated_sot_attrs[key]), EXAMPLE[key]
                )
            else:
                assert_attributes_equal(self, getattr(sot, key), value)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import exceptions
from openstack import resource
from openstack import utils


class Connection(resource.Resource):
    base_path = '/vpc-endpoint-services/%(endpoint_service_id)s/connections'
    resources_key = 'connections'

    # capabilities
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'id',
        'limit',
        'marker_id',
        'offset',
        'sort_dir',
        'sort_key',
    )

    # URI properties
    endpoint_service_id = resource.URI('endpoint_service_id')

    # Properties
    #: Creation time of the VPC endpoint.
    created_at = resource.Body('created_at')
    #: User's domain ID.
    domain_id = resource.Body('domain_id')
    #: ID of the VPC endpoint.
    id = resource.Body('id')
    #: Packet ID of the VPC endpoint.
    marker_id = resource.Body('marker_id', type=int)
    #: Connection status of the VPC endpoint.
    status = resource.Body('status')
    #: Update time of the VPC endpoint.
    updated_at = resource.Body('updated_at')

    def _action(self, session, action, endpoints=[]):
        """Preform actions given the message body."""
        uri = utils.urljoin(self.base_path % self._uri.attributes, 'action')
        body = {'endpoints': endpoints, 'action': action}
        response = session.post(uri, json=body)
        exceptions.raise_from_response(response)
        for raw_resource in response.json()[self.resources_key]:
            yield Connection.existing(**raw_resource)

    def accept(self, session, endpoints=[]):
        """Accept connections."""
        return self._action(session, 'receive', endpoints)

    def reject(self, session, endpoints=[]):
        """Reject Connections."""
        return self._action(session, 'reject', endpoints)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""Backup v1 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def set_attributes_for_print_detail(obj):
    info = {}  # instance.copy()
    info['id'] = obj.id
    info['name'] = obj.name
    info['instance_id'] = obj.instance_id
    info['size'] = obj.size
    info['status'] = obj.status
    info['created'] = obj.created
    info['updated'] = obj.updated
    if obj.datastore:
        info['datastore_type'] = obj.datastore['type']
        info['datastore_version'] = obj.datastore['version']

    return info


class ListBackup(command.Lister):

    _description = _("List database backups/snapshots")
    columns = ('ID', 'Name', 'instance_id', 'datastore_type',
               'datastore_version',
               'size', 'status', 'created', 'updated')

    def get_parser(self, prog_name):
        parser = super(ListBackup, self).get_parser(prog_name)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        data = client.backups()

        return (
            self.columns,
            (utils.get_dict_properties(
                set_attributes_for_print_detail(s),
                self.columns,
            ) for s in data)
        )


class CreateBackup(command.ShowOne):
    _description = _('Create Database backup')

    columns = ('ID', 'Name', 'instance_id', 'datastore_type',
               'datastore_version',
               'size', 'status', 'created', 'updated')

    def get_parser(self, prog_name):
        parser = super(CreateBackup, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID of the instance to take backup from')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            required=True,
            help=_('Name for the backup')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description for the backup')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        info = {}

        info['name'] = parsed_args.name
        if parsed_args.description:
            info['description'] = parsed_args.description

        data = client.create_backup(instance=parsed_args.instance,
                                    **info)

        return (
            self.columns,
            utils.get_dict_properties(
                set_attributes_for_print_detail(data),
                self.columns,
            )
        )


class DeleteBackup(command.Command):
    _description = _('Delete Backup')

    def get_parser(self, prog_name):
        parser = super(DeleteBackup, self).get_parser(prog_name)
        parser.add_argument(
            'backup',
            metavar='<backup>',
            nargs='+',
            help=_('ID of the backup')
        )
        return parser

    def take_action(self, parsed_args):

        if parsed_args.backup:
            client = self.app.client_manager.rds
            for bck in parsed_args.backup:
                client.delete_backup(
                    backup=bck,
                    ignore_missing=False)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.sdk.sdrs.v1 import _proxy

from openstack.tests.unit import test_proxy_base

from otcextensions.sdk.sdrs.v1 import job as _job
from otcextensions.sdk.sdrs.v1 import active_domains as _active_domains
from otcextensions.sdk.sdrs.v1 import protection_group as _protection_group
from otcextensions.sdk.sdrs.v1 import protected_instance as \
    _protected_instance
from otcextensions.sdk.sdrs.v1 import replication_pair as _replication_pair
from otcextensions.sdk.sdrs.v1 import dr_drill as _dr_drill
from otcextensions.sdk.sdrs.v1 import task_center as _task_center
from otcextensions.sdk.sdrs.v1 import quota as _quota


class TestSDRSProxy(test_proxy_base.TestProxyBase):

    def setUp(self):
        super(TestSDRSProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestSDRSJob(TestSDRSProxy):

    def test_get_job(self):
        self.verify_get(self.proxy.get_job, _job.Job)


class TestSDRSActiveDomains(TestSDRSProxy):

    def test_get_domains(self):
        self.verify_list(self.proxy.get_domains,
                         _active_domains.ActiveDomains)


class TestSDRSProtectionGroup(TestSDRSProxy):

    def test_protection_group_create(self):
        self.verify_create(self.proxy.create_protection_group,
                           _protection_group.ProtectionGroup,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_protection_groups(self):
        self.verify_list(self.proxy.protection_groups,
                         _protection_group.ProtectionGroup)

    def test_protection_group_get(self):
        self.verify_get(self.proxy.get_protection_group,
                        _protection_group.ProtectionGroup)

    def test_protection_group_delete(self):
        self.verify_delete(self.proxy.delete_protection_group,
                           _protection_group.ProtectionGroup,
                           True)

    def test_protection_group_find(self):
        self.verify_find(self.proxy.find_protection_group,
                         _protection_group.ProtectionGroup)

    def test_protection_group_update(self):
        group = _protection_group.ProtectionGroup()
        name = 'name'
        self._verify(
            'openstack.proxy.Proxy._update',
            self.proxy.update_protection_group,
            method_args=[group, name],
            expected_args=[_protection_group.ProtectionGroup, group],
            expected_kwargs={
                'name': name
            }
        )

    def test_enable_protection(self):
        group = _protection_group.ProtectionGroup()
        self._verify(
            'otcextensions.sdk.sdrs.v1.'
            'protection_group.ProtectionGroup.enable_protection_group',
            self.proxy.enable_protection,
            method_args=[group],
            expected_args=[self.proxy],
            expected_kwargs={
                'protection_group': group.id
            }
        )

    def test_disable_protection(self):
        group = _protection_group.ProtectionGroup()
        self._verify(
            'otcextensions.sdk.sdrs.v1.protection_group.'
            'ProtectionGroup.disable_protection_group',
            self.proxy.disable_protection,
            method_args=[group],
            expected_args=[self.proxy],
            expected_kwargs={
                'protection_group': group.id
            }
        )

    def test_perfrom_failover(self):
        group = _protection_group.ProtectionGroup()
        self._verify(
            'otcextensions.sdk.sdrs.v1.protection_group.'
            'ProtectionGroup.perform_failover',
            self.proxy.perform_failover,
            method_args=[group],
            expected_args=[self.proxy],
            expected_kwargs={
                'protection_group': group.id
            }
        )

    def test_perfrom_planned_failover(self):
        group = _protection_group.ProtectionGroup()
        priority_station = 'target'
        self._verify(
            'otcextensions.sdk.sdrs.v1.protection_group.'
            'ProtectionGroup.perform_planned_failover',
            self.proxy.perform_planned_failover,
            method_args=[group, priority_station],
            expected_args=[self.proxy],
            expected_kwargs={
                'protection_group': group.id,
                'priority_station': priority_station
            }
        )


class TestSDRSProtedInstance(TestSDRSProxy):

    def test_protected_instance_create(self):
        self.verify_create(self.proxy.create_protected_instance,
                           _protected_instance.ProtectedInstance,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_protected_instance_delete(self):
        protected_instance = _protected_instance.ProtectedInstance()
        delete_target_server = True
        self._verify(
            mock_method='otcextensions.sdk.sdrs.v1.'
                        'protected_instance.ProtectedInstance.delete',
            test_method=self.proxy.delete_protected_instance,
            method_args=[protected_instance, delete_target_server],
            expected_args=[self.proxy],
            expected_kwargs={
                'delete_target_server': delete_target_server,
                'delete_target_eip': False
            }
        )

    def test_protected_instance(self):
        self.verify_list(self.proxy.protected_instances,
                         _protected_instance.ProtectedInstance)

    def test_protected_instance_get(self):
        self.verify_get(self.proxy.get_protected_instance,
                        _protected_instance.ProtectedInstance)

    def test_protected_instance_update(self):
        protected_instance = _protected_instance.ProtectedInstance()
        name = 'name'
        self._verify(
            'openstack.proxy.Proxy._update',
            self.proxy.update_protected_instance,
            method_args=[protected_instance, name],
            expected_args=[_protected_instance.ProtectedInstance,
                           protected_instance],
            expected_kwargs={
                'name': name
            }
        )

    def test_protected_instance_find(self):
        self.verify_find(self.proxy.find_protected_instance,
                         _protected_instance.ProtectedInstance)

    def test_attach_replication_pair(self):
        protected_instance = _protected_instance.ProtectedInstance()
        replication = _replication_pair.ReplicationPair()
        device = 'device_name'
        self._verify(
            mock_method='otcextensions.sdk.sdrs.v1.protected_instance.'
                        'ProtectedInstance.attach_pair',
            test_method=self.proxy.attach_replication_pair,
            method_args=[protected_instance, replication, device],
            expected_args=[self.proxy],
            expected_kwargs={
                'protected_instance': protected_instance.id,
                'replication_id': replication.id,
                'device': device
            }
        )

    def test_detach_replication_pair(self):
        protected_instance = _protected_instance.ProtectedInstance()
        replication = _replication_pair.ReplicationPair()
        self._verify(
            mock_method='otcextensions.sdk.sdrs.v1.protected_instance.'
                        'ProtectedInstance.detach_pair',
            test_method=self.proxy.detach_replication_pair,
            method_args=[protected_instance, replication],
            expected_args=[self.proxy],
            expected_kwargs={
                'protected_instance': protected_instance.id,
                'replication_id': replication.id
            }
        )

    def test_add_nic(self):
        protected_instance = _protected_instance.ProtectedInstance()
        subnet_id = 'subnet_id'
        security_groups = ['security_group_id_1', 'security_group_id_2']
        ip_address = 'ip_address'
        self._verify(
            mock_method='otcextensions.sdk.sdrs.v1.protected_instance.'
                        'ProtectedInstance.add_nic',
            test_method=self.proxy.add_nic,
            method_args=[protected_instance, subnet_id,
                         security_groups, ip_address],
            expected_args=[self.proxy],
            expected_kwargs={
                'protected_instance': protected_instance.id,
                'subnet_id': subnet_id,
                'security_groups': security_groups,
                'ip_address': ip_address
            }
        )

    def test_delete_nic(self):
        protected_instance = _protected_instance.ProtectedInstance()
        nic_id = 'nic_id'
        self._verify(
            mock_method='otcextensions.sdk.sdrs.v1.protected_instance.'
                        'ProtectedInstance.delete_nic',
            test_method=self.proxy.delete_nic,
            method_args=[protected_instance, nic_id],
            expected_args=[self.proxy],
            expected_kwargs={
                'protected_instance': protected_instance.id,
                'nic_id': nic_id
            }
        )

    def test_protected_instance_modify(self):
        protected_instance = _protected_instance.ProtectedInstance()
        flavor = 'flavor_id'
        production_flavor = 'production_flavor'
        self._verify(
            mock_method='otcextensions.sdk.sdrs.v1.protected_instance.'
                        'ProtectedInstance.modify_instance',
            test_method=self.proxy.modify_protected_instance,
            method_args=[protected_instance, flavor, production_flavor],
            expected_args=[self.proxy],
            expected_kwargs={
                'protected_instance': protected_instance.id,
                'flavor': flavor,
                'production_flavor': production_flavor,
                'dr_flavor': None
            }
        )


class TestSDRSReplicationPair(TestSDRSProxy):

    def test_replication_pair_create(self):
        self.verify_create(self.proxy.create_replication_pair,
                           _replication_pair.ReplicationPair,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_replication_pair_delete(self):
        replication_pair = _replication_pair.ReplicationPair()
        server_group_id = 'server_group_id'
        delete_target_volume = True
        self._verify(
            mock_method='otcextensions.sdk.sdrs.v1.replication_pair.'
                        'ReplicationPair.delete',
            test_method=self.proxy.delete_replication_pair,
            method_args=[replication_pair,
                         server_group_id,
                         delete_target_volume],
            expected_args=[self.proxy],
            expected_kwargs={
                'server_group_id': server_group_id,
                'delete_target_volume': delete_target_volume
            }
        )

    def test_replication_pair(self):
        self.verify_list(self.proxy.replication_pairs,
                         _replication_pair.ReplicationPair)

    def test_replication_pair_get(self):
        self.verify_get(self.proxy.get_replication_pair,
                        _replication_pair.ReplicationPair)

    def test_replication_pair_find(self):
        self.verify_find(self.proxy.find_replication_pair,
                         _replication_pair.ReplicationPair)

    def test_replication_pair_expand(self):
        replication = _replication_pair.ReplicationPair(id='replication')
        new_size = 100
        self._verify(
            mock_method='otcextensions.sdk.sdrs.v1.replication_pair.'
                        'ReplicationPair.expand_replication',
            test_method=self.proxy.expand_replication_pair,
            method_args=[replication, new_size],
            expected_args=[self.proxy],
            expected_kwargs={
                'replication': replication.id,
                'new_size': new_size
            }
        )

    def test_replication_pair_update(self):
        replication = _replication_pair.ReplicationPair()
        name = 'name'
        self._verify(
            'openstack.proxy.Proxy._update',
            self.proxy.update_replication_pair,
            method_args=[replication, name],
            expected_args=[_replication_pair.ReplicationPair, replication],
            expected_kwargs={
                'name': name
            }
        )


class TestSDRSDRDrill(TestSDRSProxy):

    def test_dr_drill_create(self):
        self.verify_create(self.proxy.create_dr_drill,
                           _dr_drill.DRDrill,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_dr_drill_delete(self):
        self.verify_delete(self.proxy.delete_dr_drill,
                           _dr_drill.DRDrill)

    def test_dr_drill_list(self):
        self.verify_list(self.proxy.dr_drills,
                         _dr_drill.DRDrill)

    def test_dr_drill_get(self):
        self.verify_get(self.proxy.get_dr_drill,
                        _dr_drill.DRDrill)

    def test_dr_drill_find(self):
        self.verify_find(self.proxy.find_dr_drill,
                         _dr_drill.DRDrill)

    def test_dr_drill_update(self):
        drill = _dr_drill.DRDrill()
        name = 'name'
        self._verify(
            'openstack.proxy.Proxy._update',
            self.proxy.update_dr_drill,
            method_args=[drill, name],
            expected_args=[_dr_drill.DRDrill, drill],
            expected_kwargs={
                'name': name
            }
        )


class TestSDRSTaskCenter(TestSDRSProxy):

    def test_failed_tasks(self):
        self.verify_list(self.proxy.failed_tasks,
                         _task_center.FailedTask)

    def test_failed_task_delete(self):
        self.verify_delete(self.proxy.delete_failed_task,
                           _task_center.FailedTask)

    def test_delete_all_failed_tasks(self):
        endpoint = self.proxy.get_endpoint()
        self._verify(
            mock_method='otcextensions.sdk.sdrs.v1.task_center.'
                        'FailedTask.delete_all_tasks',
            test_method=self.proxy.delete_all_failed_tasks,
            method_args=[],
            expected_args=[self.session, endpoint]
        )

    def test_delete_protection_group_tasks(self):
        endpoint = self.proxy.get_endpoint()
        group = _protection_group.ProtectionGroup()
        self._verify(
            mock_method='otcextensions.sdk.sdrs.v1.task_center.'
                        'FailedTask.delete_protection_tasks',
            test_method=self.proxy.delete_protection_group_tasks,
            method_args=[group],
            expected_args=[self.session, endpoint, group.id]
        )


class TestSDRSQuotas(TestSDRSProxy):

    def test_quotas(self):
        self.verify_list(self.proxy.quotas,
                         _quota.Quota)



# Licensed under the Apache License, Version 2.0 (the "License"); you may

# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
# from botocore.exceptions import ClientError
import base64
import hashlib
import xml.etree.ElementTree as ET
from io import BufferedReader

from openstack import _log
from openstack import exceptions
from openstack import resource

# from otcextensions.i18n import _
from otcextensions.sdk.obs.v1 import _base

_logger = _log.setup_logging('openstack')


class Object(_base.BaseResource):
    _custom_metadata_prefix = "x-amz-meta-"
    base_path = '/'

    allow_create = True
    allow_get = True
    allow_commit = True
    allow_delete = True
    allow_list = True
    allow_head = True

    resources_key = ''
    resource_key = 'Contents'

    _query_mapping = resource.QueryParameters(
        'prefix', 'delimiter',
        'limit',
        prefix='prefix',
        delimiter='delimiter',
        limit='max-keys'
    )

    # Data to be passed during a POST call to create an object on the server.
    data = None

    # URL parameters
    #: The unique name for the container.
    container = resource.URI("container")
    #: The unique name for the object.
    name = resource.Body('Key', alternate_id=True)
    #: The date and time that the object was created or the last
    #: time that the metadata was changed.
    last_modified = resource.Body('LastModified')
    #: size of the response body. Instead it contains the size of
    #: the object, in bytes.
    content_length = resource.Body('Size', type=int)
    # Headers for requests
    #: private, public-read, public-read-write, authenticated-read
    #: bucket-owner-read, bucket-owner-full-control
    acl = resource.Header('x-amz-acl')

    accept_ranges = resource.Header('Accept-Ranges')
    #: The MD5 digest string of the message body is calculated according
    #: to the RFC 1864 standard. That is, calculate the 128-bit binary array
    #: (the message header data encrypted with MD5) first,
    #: and then use Base 64 encoding to convert the binary data to
    #: a character string.
    content_md5 = resource.Header('Content-MD5', type=str)
    #: Indicates the content type of a requested resource, for example,
    #: text/plain.
    content_type = resource.Header('Content-Type', type=str)
    #: Indicates the hash value of an object.
    #: The entity tag (ETag) only reflects changes to the contents
    #: of an object, not its metadata.
    etag = resource.Header('ETag', type=str)
    #: Indicates the value created by OBS to uniquely identify a request.
    #: OBS uses this value to troubleshoot faults.
    request_id = resource.Header('x-amz-request-id', type=str)
    #: Indicates a special token that helps OBS troubleshoot faults.
    request_id_2 = resource.Header('x-amz-id-2', type=str)
    #: Indicates that SSE-KMS is used.
    #: Example: x-amz-server-side-encryption:aws:kms
    sse = resource.Header('x-amz-server-side-encryption')
    #: Indicates the master key ID. This header is used in SSE-KMS mode.
    #: If the customer does not provide the master key,
    #: the default master key will be used.
    sse_key_id = resource.Header('x-amz-server-side-encryption-aws-kms-key-id')
    #: Indicates a decryption algorithm. The header is used in SSE-C mode.
    #: Constraints: This header must be used together with
    #: x-amz-server-side-encryption-customer-key and
    #: x-amz-server-side-encryption-customer-key-MD5.
    sse_algorithm = resource.Header(
        'x-amz-server-side-encryption-customer-algorithm'
    )
    #: Indicates a key used to decrypt objects.
    #: The header is used in SSE-C mode.
    #: Constraints: This header is a base64-encoded 256-bit or 512-bit key and
    #: must be used together with
    # x-amz-server-side-encryption-customer-algorithm and
    # x-amz-server-side-encryption-customer-key-MD5
    sse_key = resource.Header('x-amz-server-side-encryption-customer-key')
    #: Indicates the MD5 value of a key used to decrypt objects.
    #: The header is used in SSE-C mode.
    #: The MD5 value is used to check whether any error
    #: occurs during the transmission of the key.
    #: Constraints: This header is a base64-encoded 128-bit MD5 value and
    #: must be used together with
    #: x-amz-server-side-encryption-customer-algorithm and
    #: x-amz-server-side-encryption-customer-key.
    sse_key_md5 = resource.Header(
        'x-amz-server-side-encryption-customer-key-MD5'
    )
    #: When creating an object, you can add this header in the request
    #: to set the storage class of the object. If you do not add this header,
    #: the object will use the default storage class of the bucket.
    #: Note: The storage class can be STANDARD (OBS Standard),
    #: STANDARD_IA (OBS Warm), or GLACIER (OBS Cold).
    #: Note that the three storage class values are case-sensitive.
    storage_class = resource.Header('x-amz-storage-class')
    #: Server name
    server = resource.Header('Server', type=str)
    #: If a bucket is configured as a website, redirects requests
    #: for this object to another object in the same bucket or to
    #: an external URL.
    #: OBS stores the value of this header in the object metadata.
    website_redirect = resource.Header('x-amz-website-redirect-location')

    #: Obtains the specified range bytes of an object.
    #: The value is a range starting from 0 to maximum object length minus one.
    #: If the range is invalid, all object data is returned.
    range = resource.Header("range", type=str)
    #: Returns the object only if it has been modified since
    #: the time specified by this header,
    #: otherwise 304 Not Modified is returned.
    if_modified_since = resource.Header("if-modified-since", type=str)
    #: Returns the object only if it has not been modified since
    #: the time specified by this header,
    #: otherwise 412 Precondition Failed is returned.
    #: http://www.ietf.org/rfc/rfc2616.txt.
    if_unmodified_since = resource.Header("if-unmodified-since", type=str)
    #: Returns the object only if its ETag is the same
    #: as the one specified by this header,
    #: otherwise 412 Precondition Failed is returned.
    #: http://www.ietf.org/rfc/rfc2616.txt.
    if_match = resource.Header("if-match", type=list)
    #: Returns the object only if its ETag is different from the one
    #: specified by this header,
    #: otherwise 304 Not Modified is returned.
    if_none_match = resource.Header("if-none-match", type=list)
    #: Indicates an origin specified by a pre-request.
    #: Generally, it is a domain name
    origin = resource.Header("Origin", type=bool)

    def __init__(self, data=None, **attrs):
        super(_base.BaseResource, self).__init__(**attrs)
        self.data = data

    def _translate_response(self, response, has_body=True, error_message=None,
                            resource_response_key=None):
        """Given a KSA response, inflate this instance with its data

        This method updates attributes that correspond to headers
        and body on this instance and clears the dirty set.
        """
        exceptions.raise_from_response(response, error_message=response.text)
        _logger.debug(response.text)
        if response:
            if has_body:
                # TODO(agoncharov): do nothing so far. Generally need
                # to parse different responses
                pass
        headers = self._consume_header_attrs(response.headers)
        self._header.attributes.update(headers)
        self._header.clean()
        self._update_location()
        dict.update(self, self.to_dict())

    @classmethod
    def list(cls, session, paginated=False,
             endpoint_override=None, headers=None, requests_auth=None,
             **params):
        if not cls.allow_list:
            raise exceptions.MethodNotSupported(cls, "list")

        cls._query_mapping._validate(params, base_path=cls.base_path)
        query_params = cls._query_mapping._transpose(params, cls)
        uri = cls.base_path % params

        # Build additional arguments to the GET call
        get_args = cls._prepare_override_args(
            endpoint_override=endpoint_override,
            additional_headers=headers)

        while uri:

            response = session.get(
                uri,
                params=query_params.copy(),
                requests_auth=requests_auth,
                **get_args
            )

            uri = None
            next_params = {}

            root = ET.fromstring(response.content)

            if root.tag != ET.QName(cls.OBS_NS, 'ListBucketResult'):
                _logger.warn('Namespace in the response does not match '
                             'expectation')
                cls.OBS_NS = root.tag.split('}', 1)[0][1:]

            for element in root:

                if element.tag == ET.QName(cls.OBS_NS, cls.resource_key):
                    # Convert XML part into dict
                    dict_raw_resource = cls.etree_to_dict(element)
                    # extract resource data
                    dict_resource = dict_raw_resource[cls.resource_key]
                    value = cls.existing(**dict_resource)
                    yield value

                elif element.tag == ET.QName(cls.OBS_NS, 'NextMarker'):
                    next_params['marker'] = element.text

            if 'marker' in next_params:
                uri = cls.base_path % params
                query_params.update(next_params)

        return

    def create(self, session, prepend_key=True,
               endpoint_override=None, headers=None, requests_auth=None):

        if not self.allow_create:
            raise exceptions.MethodNotSupported(self, 'create')

        session = self._get_session(session)

        if not self.content_md5 and self.data and\
                not isinstance(self.data, BufferedReader):
            md5 = hashlib.md5()
            md5.update(str.encode(self.data))
            self.content_md5 = base64.b64encode(md5.digest()).decode()

        request = self._prepare_request(
            requires_id=True,
            prepend_key=prepend_key)

        if self.id[-1] == "/":
            request.url += "/"

        req_args = self._prepare_override_args(
            endpoint_override=endpoint_override,
            request_headers=request.headers,
            additional_headers=headers,
            requests_auth=requests_auth)

        response = session.put(
            request.url,
            data=self.data,
            **req_args)
        self._translate_response(response)
        return self

    def delete(self, session, error_message=None,
               endpoint_override=None, headers=None,
               requests_auth=None, params=None):

        if not self.allow_delete:
            raise exceptions.MethodNotSupported(self, "delete")

        request = self._prepare_request()
        if self.id[-1] == "/":
            request.url += "/"

        session = self._get_session(session)

        delete_args = self._prepare_override_args(
            endpoint_override=endpoint_override,
            request_headers=request.headers,
            additional_headers=headers,
            requests_auth=requests_auth)
        if params:
            delete_args['params'] = params

        response = session.delete(request.url,
                                  **delete_args)
        kwargs = {}
        if error_message:
            kwargs['error_message'] = error_message

        self._translate_response(response, has_body=False, **kwargs)
        return self

    def download(self, session, filename=None,
                 endpoint_override=None, requests_auth=None):

        session = self._get_session(session)

        request = self._prepare_request(requires_id=True)

        req_args = self._prepare_override_args(
            endpoint_override=endpoint_override,
            request_headers=request.headers,
            requests_auth=requests_auth)

        response = session.get(
            request.url,
            **req_args)
        self._translate_response(response)

        _logger.debug(response.content)

        with open(filename, 'wb') as f:
            f.write(response.content)

        return

    @staticmethod
    def initiate_multipart_upload(proxy, endpoint, name, **params):
        response = proxy.post(url=f'/{name}?uploads',
                              endpoint_override=endpoint,
                              **params)
        dict_resource = {}
        root = ET.fromstring(response.content)
        for element in root:
            dict_raw_resource = _base.BaseResource.etree_to_dict(element)
            dict_resource.update(dict_raw_resource)
        return dict_resource['UploadId']

    @staticmethod
    def get_parts(proxy, endpoint, requests_auth):
        response = proxy.get(endpoint, requests_auth=requests_auth)
        dict_resource = {}
        root = ET.fromstring(response.content)
        for element in root:
            dict_raw_resource = _base.BaseResource.etree_to_dict(element)
            if element.tag == 'Part':
                dict_resource.setdefault('Parts', []).\
                    append(dict_raw_resource['Part'])
                continue
            dict_resource.update(dict_raw_resource)
        return dict_resource

    @staticmethod
    def complete_multipart_upload(
            proxy, endpoint, upload_id, data, headers, requests_auth):
        url = f'{endpoint}?uploadId={upload_id}'
        root = ET.Element("CompleteMultipartUpload")
        for item in data:
            part = ET.SubElement(root, "Part")
            ET.SubElement(part, 'PartNumber').text = item['PartNumber']
            ET.SubElement(part, 'ETag').text = item['ETag']
        tree = ET.ElementTree(root)
        data = ET.tostring(tree.getroot()).decode()
        return proxy.post(url, data=data,
                          headers=headers, requests_auth=requests_auth)
